/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _EBPatch_Polytropic_Factory_H_
#define _EBPatch_Polytropic_Factory_H_

#include "EBPatchPolytropic.H"
#include "EBPatchGodunovFactory.H"

///
/**
 */
class EBPatchPolytropicFactory: public EBPatchGodunovFactory
{
public:
  ///
  /**
   */
  virtual ~EBPatchPolytropicFactory();

  ///
  /**
   */
  EBPatchGodunov* create() const;

                           
  ///
  /**
   */
  EBPatchPolytropicFactory(const EBPhysIBCFactory* const a_bc,
                           const Real&                   a_gamma,
                           const bool&                   a_useFourthOrderSlopes,
                           const bool&                   a_useFlattening,
                           const bool&                   a_useArtificialVisc,
                           const bool&                   a_useLimiting,
                           const bool&                   a_doRZCoords);

protected:
  const EBPhysIBCFactory* m_bcFactoryPtr;
  Real              m_gamma;
  bool              m_useFourthOrderSlopes;
  bool              m_useFlattening;
  bool              m_useArtificialVisc;
  bool              m_doRZCoords;
  bool              m_useLimiting;

private:

  //disallowed for all the usual reasons
  EBPatchPolytropicFactory()
  {
    MayDay::Error("invalid operator");
  }
  void operator=(const EBPatchPolytropicFactory& a_input)
  {
    MayDay::Error("invalid operator");
  }
  EBPatchPolytropicFactory(const EBPatchPolytropicFactory& a_input)
  {
    MayDay::Error("invalid operator");
  }
};
#endif
