/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "EBPatchPolytropicFactory.H"
/******************/
/******************/
EBPatchPolytropicFactory::
EBPatchPolytropicFactory(const EBPhysIBCFactory* const a_bcFactoryPtr,
                         const Real&                   a_gamma,
                         const bool&                   a_useFourthOrderSlopes,
                         const bool&                   a_useFlattening,
                         const bool&                   a_useArtificialVisc,
                         const bool&                   a_useLimiting,
                         const bool&                   a_doRZCoords)
  :EBPatchGodunovFactory()
{
  m_useLimiting          = a_useLimiting;
  m_gamma                = a_gamma;               
  m_useFourthOrderSlopes = a_useFourthOrderSlopes;
  m_bcFactoryPtr         = a_bcFactoryPtr;
  m_useFlattening        = a_useFlattening;       
  m_useArtificialVisc    = a_useArtificialVisc;       
  m_doRZCoords           = a_doRZCoords;
}

/******************/
/******************/
EBPatchPolytropicFactory::
~EBPatchPolytropicFactory()
{;}
/******************/
/******************/
EBPatchGodunov* 
EBPatchPolytropicFactory::
create() const
{
  EBPatchPolytropic* retval= new EBPatchPolytropic();
  retval->setSlopeParameters(m_useFourthOrderSlopes,
                             m_useFlattening,
                             m_useLimiting);
  retval->artificialViscosity(m_useArtificialVisc);
  retval->setEBPhysIBC(*m_bcFactoryPtr);
  retval->setGamma(m_gamma);
  retval->doRZCoords(m_doRZCoords);
  return static_cast<EBPatchGodunov*>(retval);
}
/******************/
/******************/


                           
