/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef  _EB_PlanarShockIBC_H__
#define  _EB_PlanarShockIBC_H__

#include  <iostream>

#include "LevelData.H"
#include "FArrayBox.H"
#include "Vector.H"
#include "RealVect.H"
#include "AMRIO.H"

#include "EBPhysIBC.H"
#include "EBPlanarShockF_F.H"
#include "EBSolidF_F.H"

class EBPlanarShockIBC : public EBPhysIBC
{
public:
  virtual ~EBPlanarShockIBC();

  EBPlanarShockIBC(const Real&     a_gamma,
                   const Real&     a_ms,
                   const Real&     a_center,
                   const int&      a_shocknorm,
                   const bool&     a_shockbackward,
                   const bool&     a_doRZCoords);

  ///
  void define(const ProblemDomain&  a_domain,
              const Real& a_dx);

  ///  For every box in this level, this function is called.
  void fluxBC(EBFaceFAB&            a_flux,
              const EBCellFAB&      a_Wcenter,
              const EBCellFAB&      a_Wextrap,
              const Side::LoHiSide& a_sd,
              const Real&           a_time,
              const EBISBox&        a_ebisBox,
              const Box&            a_box,
              const int&            a_dir);
  
  /// Initialize
  void initialize(LevelData<EBCellFAB>& a_conState,
                  const EBISLayout& a_ebisl) const;

  /// Set boundary slopes
  /**
     The boundary slopes in a_dW are already set to one sided difference
     approximations.  If this function doesn't change them they will be
     used for the slopes at the boundaries.
  */
  void setBndrySlopes(EBCellFAB&       a_deltaPrim,
                      const EBCellFAB& a_primState,
                      const EBISBox&   a_ebisBox,
                      const Box&       a_box,
                      const int&       a_dir);

protected:
  bool m_isFortranCommonSet;
  bool m_isDefined;
  Real m_gamma;
  Real m_ms;
  Real m_center;
  int  m_shocknorm;
  bool m_shockbackward;
  bool m_doRZCoords;

  Real m_dx;
  ProblemDomain  m_domain;

private:
  EBPlanarShockIBC()
  {
    MayDay::Error("invalid operator");
  }
  
  //disallowed for all the usual reasons
  void operator=(const EBPlanarShockIBC& a_input)
  {
    MayDay::Error("invalid operator");
  }
  EBPlanarShockIBC(const EBPlanarShockIBC& a_input)
  {
    MayDay::Error("invalid operator");
  }

    
};

#endif
