/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "ModianoIBCFactory.H"
#include "ModianoIBC.H"
#include "PolyGeom.H"
/******************/
/******************/
ModianoIBCFactory::ModianoIBCFactory(const Real&     a_gamma,
                                     const Real&     a_waveAmp,
                                     const Real&     a_waveWidth,
                                     const RealVect& a_center,
                                     const RealVect& a_waveDir,
                                     bool     a_freeStreamOnly,
                                     bool     a_useNegWave)
  :EBPhysIBCFactory()
{
  m_gamma       = a_gamma       ;
  m_waveAmp     = a_waveAmp     ;
  m_waveWidth   = a_waveWidth   ;
  m_waveDir     = a_waveDir     ;
  m_center      = a_center      ;
  m_freeStreamOnly = a_freeStreamOnly;
  m_useNegWave = a_useNegWave;
}

/******************/
/******************/

ModianoIBCFactory::
~ModianoIBCFactory()
{;}
/******************/
/******************/
EBPhysIBC*
ModianoIBCFactory::
create() const
{
  ModianoIBC* retval = 
    new ModianoIBC(m_gamma,
                   m_waveAmp,
                   m_waveWidth,
                   m_center,
                   m_waveDir,
                   m_freeStreamOnly,
                   m_useNegWave);
  return static_cast<EBPhysIBC*>(retval);
}
/******************/
/******************/

