/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _SQUARECYLINDERIF_H_
#define _SQUARECYLINDERIF_H_

#include "MayDay.H"
#include "RealVect.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function specifies a square (infinite) cylinder
    around the x axis
 */
class SquareCylinderIF: public BaseImplicitFunction
{
public:
  ///
  /**
      Constructor specifying cylinder radius (a_radius), direction of the
      axis (a_direction), a point on the axis (a_point), and whether the
      domain is on the inside (a_inside).
   */
  SquareCylinderIF(const Real&     a_radius,
                   const bool&     a_inside);

  /// Copy constructor
  SquareCylinderIF(const SquareCylinderIF& a_inputIF);

  /// Destructor
  virtual ~SquareCylinderIF();

  ///
  /**
      Return the parameter information
   */
  virtual void GetParams(Real&     a_radius,
                         bool&     a_inside) const;

  ///
  /**
      Set the parameter information
   */
  virtual void SetParams(const Real&     a_radius,
                         const bool&     a_inside);

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  Real     m_radius;      // radius
  bool     m_inside;      // inside flag


private:
  SquareCylinderIF()
  {
    MayDay::Error("SquareCylinderIF uses strong construction");
  }

  void operator=(const SquareCylinderIF& a_inputIF)
  {
    MayDay::Error("SquareCylinderIF doesn't allow assignment");
  }
};

#endif
