/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "PolyGeom.H"

#include "SquareCylinderIF.H"

SquareCylinderIF::SquareCylinderIF(const Real&     a_radius,
                                   const bool&     a_inside)
{
  // Remember the parameters
  m_radius = a_radius;
  m_inside = a_inside;
}

SquareCylinderIF::SquareCylinderIF(const SquareCylinderIF& a_inputIF)
{
  // Remember the parameters
  m_radius = a_inputIF.m_radius;
  m_inside = a_inputIF.m_inside;
}

SquareCylinderIF::~SquareCylinderIF()
{
}

void SquareCylinderIF::GetParams(Real&     a_radius,
                                 bool&     a_inside) const
{
  // Copy parameter information over
  a_radius = m_radius;
  a_inside = m_inside;
}

void SquareCylinderIF::SetParams(const Real&     a_radius,
                                 const bool&     a_inside)
{
  // Set parameter information
  m_radius = a_radius;
  m_inside = a_inside;
}

Real SquareCylinderIF::value(const RealVect& a_point) const
{
  Real dist = 0.0;
  //dist is max dist along coord line from x axis (y = 0, z = 0 line)
  for(int idir = 1; idir < SpaceDim; idir++)
    {
      Real tandist  =  Abs(a_point[idir]);
      dist = Max(dist, tandist);
    }
  
  assert(m_radius > 0);
  Real retval = dist - m_radius;

  // Change the sign to change inside to outside
  if (!m_inside)
  {
    retval = -retval;
  }

  return retval;
}

BaseImplicitFunction* SquareCylinderIF::newImplicitFunction() const
{
  SquareCylinderIF* squareCylinderPtr = new SquareCylinderIF(m_radius,
                                                             m_inside);

  return static_cast<BaseImplicitFunction*>(squareCylinderPtr);
}
