/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _WEDGEIF_H_
#define _WEDGEIF_H_

#include "MayDay.H"
#include "RealVect.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function specifies a square (infinite) cylinder
    around the x axis
 */
class WedgeIF: public BaseImplicitFunction
{
public:
  ///
  /**
     !                        depVar
     !            \            ^
     !           / \           |
     !          /   \          |--->indVar
     !         /     \
     !        /       \
     !-------/         \------------
     center is the point in indvar 
   */
  WedgeIF(const RealVect& a_center,
          const Real&     a_slope,
          const int&      a_depVar,
          const int&      a_indVar);

  /// Copy constructor
  WedgeIF(const WedgeIF& a_inputIF);

  /// Destructor
  virtual ~WedgeIF();


  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  RealVect m_center;
  Real     m_slope;
  int      m_depVar;
  int      m_indVar;

private:
  WedgeIF()
  {
    MayDay::Error("WedgeIF uses strong construction");
  }

  void operator=(const WedgeIF& a_inputIF)
  {
    MayDay::Error("WedgeIF doesn't allow assignment");
  }
};

#endif
