/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "PolyGeom.H"

#include "WedgeIF.H"

WedgeIF::WedgeIF(const RealVect& a_center,
                 const Real&     a_slope,
                 const int&      a_depVar,
                 const int&      a_indVar)
{
  // Remember the parameters
  m_center= a_center;
  m_slope = a_slope ;
  m_depVar = a_depVar ;
  m_indVar = a_indVar ;
}

WedgeIF::WedgeIF(const WedgeIF& a_inputIF)
{
  // Remember the parameters
  m_center= a_inputIF.m_center;
  m_slope = a_inputIF.m_slope ;
  m_depVar = a_inputIF.m_depVar ;
  m_indVar = a_inputIF.m_indVar ;
}

WedgeIF::~WedgeIF()
{
}


Real WedgeIF::value(const RealVect& a_point) const
{
  assert(m_depVar >= 0);
  assert(m_depVar < SpaceDim);
  assert(m_indVar >= 0);
  assert(m_indVar < SpaceDim);
  assert(m_indVar != m_depVar);

  Real indDist = a_point[m_indVar] - m_center[m_indVar];
  Real funcVal = m_center[m_depVar] - m_slope*Abs(indDist);
  Real retval  = funcVal - a_point[m_depVar];

  return retval;
}

BaseImplicitFunction* WedgeIF::newImplicitFunction() const
{
  WedgeIF* wedgePtr = new WedgeIF(m_center,
                                  m_slope,
                                  m_depVar,
                                  m_indVar);

  return static_cast<BaseImplicitFunction*>(wedgePtr);
}
