/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _EBSIMPLESOLVER_H_
#define _EBSIMPLESOLVER_H_

#include "LevelData.H"
#include "LinearSolver.H"
#include "MultiGrid.H"

#include "EBCellFAB.H"

class EBSimpleSolver : public LinearSolver<LevelData<EBCellFAB> >
{
public:
  EBSimpleSolver();

  virtual ~EBSimpleSolver();

  virtual void setHomogeneous(bool a_homogeneous);

  virtual void define(LinearOp<LevelData<EBCellFAB> >* a_operator,
                      bool                             a_homogeneous);

  virtual void setNumSmooths(const int& a_numSmooths);

  virtual void solve(LevelData<EBCellFAB>&       a_phi,
                     const LevelData<EBCellFAB>& a_rhs);

private:
  bool m_isDefined;

  MGLevelOp<LevelData<EBCellFAB> >* m_operator;

  bool m_homogeneous;
  int  m_numSmooths;
};

#endif
