#!/bin/csh -f
#
# Spatial dimension
#
setenv dimnum 2
if ($#argv >= 1) then
  if ("$1" == "-h") then
    echo "Usage:  runsome.csh [dim [dir [maxsize [startsize startsingx startsingy startsingz]]]]"
    exit
  else
    setenv dimnum ${1}
  endif
endif
setenv dim ${dimnum}d
#
# Directory for results
#
setenv dir results
if ($#argv >= 2) then
  setenv dir $2
endif
#
# Maximum size
#
if ($#argv >= 3) then
  setenv sizemax $3
endif
#
# Set starting size and singularity indices
#
setenv ssize 32
setenv ssingx 1
setenv ssingy 1
setenv ssingz 1
if ($#argv >= 4) then
  setenv ssize $4
  setenv ssingx $5
  setenv ssingy $6
  setenv ssingz $7
endif
#
# Lists of singularity coordinates
#
setenv singxall "-0.43 0.52 1.61"
setenv singyall "-0.51 0.63 1.42"
setenv singzall "-0.62 0.41 1.53"
#
# Parameters for linear halfspace
#
setenv px 0.43
setenv py 0.52
setenv pz 0.61
setenv nx 1.13
setenv ny 2.22
setenv nz 3.31
#
# Parameters for sphere
#
setenv cx     0.29
setenv cy     0.24
setenv cz     0.19
setenv radius 0.695
#
# Parameters for polar function
#
setenv pcentx  0.43
setenv pcenty  0.52
setenv pcentz  0.57
setenv prad    0.291
setenv ppert  -0.104
setenv pfreq   6
#
# Exterior of a polar function
# Dirichlet domain BC, 1st order Dirichlet EB BC
# Use W-cycle
#
setenv name  polardir1
setenv ebbc  1
setenv dombc 1
setenv cycle 2
setenv geom  2
setenv int   0
runone.csh
#
# Linear halfspace 
# Dirichlet domain BC, 2nd order Dirichlet EB BC
# Use V-cycle
#
setenv name  halfdir2
setenv ebbc  2
setenv dombc 1
setenv cycle 1
setenv geom  0
setenv int   0
runone.csh
#
# Interior of a circle which intersects the domain
# Dirichlet domain BC, 2nd order Dirichlet EB BC
# Use V-cycle
#
setenv name  sphdir2
setenv ebbc  2
setenv dombc 1
setenv cycle 1
setenv geom  1
setenv int   1
runone.csh
#
# Exterior of a polar function
# Dirichlet domain BC, 2nd order Dirichlet EB BC
# Use V-cycle
#
setenv name  polardir2
setenv ebbc  2
setenv dombc 1
setenv cycle 1
setenv geom  2
setenv int   0
runone.csh
#
# Linear halfspace 
# Dirichlet domain BC, Neumann EB BC
# Use V-cycle
#
setenv name  halfneu
setenv ebbc  0
setenv dombc 1
setenv cycle 1
setenv geom  0
setenv int   0
runone.csh
#
# Interior of a circle which intersects the domain
# Dirichlet domain BC, Neumann EB BC
# Use V-cycle
#
setenv name  sphneu
setenv ebbc  0
setenv dombc 1
setenv cycle 1
setenv geom  1
setenv int   1
runone.csh
#
# Exterior of a polar function
# Dirichlet domain BC, Neumann EB BC
# Use V-cycle
#
setenv name  polarneu
setenv ebbc  0
setenv dombc 1
setenv cycle 1
setenv geom  2
setenv int   0
runone.csh
#
# Generate summaries
#
geterrors.csh $dir
getrates.csh  $dir
foreach name (                 polardir1 \
              halfdir2 sphdir2 polardir2 \
              halfneu  sphneu  polarneu  \
             )
  getsummary.csh $name $dimnum $dir |& tee $dir/summary.$dim.$name
end
