/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "RealVect.H"

#include "functionsF_F.H"
#include "TrigBCFlux.H"

TrigBCFlux::TrigBCFlux()
{
  m_isDefined = false;

  m_trig = RealVect::Unit;
}

TrigBCFlux::~TrigBCFlux()
{
}

void TrigBCFlux::define(const RealVect& a_trig)
{
  m_isDefined = true;

  m_trig = a_trig;
}

Real TrigBCFlux::flux(const RealVect& a_point,
                      const RealVect& a_normal)
{
  assert(m_isDefined);

  RealVect gradient;

  FORT_GETGRADPHIPOINT(CHF_REALVECT(gradient),
                       CHF_CONST_REALVECT(m_trig),
                       CHF_CONST_REALVECT(a_point));

  Real flux = 0.0;
  for (int idir = 0; idir < SpaceDim; idir++)
  {
    flux += gradient[idir] * a_normal[idir];
  }

  return flux;
}
