#!/bin/csh -f
#
# Spatial dimension
#
setenv dimnum 2
if ($#argv >= 1) then
  if ("$1" == "-h") then
    echo "Usage:  runsome.csh [dim [dir [maxsize [startsize startfreqx startfreqy startfreqz]]]]"
    exit
  else
    setenv dimnum ${1}
  endif
endif
setenv dim ${dimnum}d
#
# Directory for results
#
setenv dir results
if ($#argv >= 2) then
  setenv dir $2
endif
#
# Maximum size
#
if ($#argv >= 3) then
  setenv sizemax $3
endif
#
# Set starting size and frequency indices
#
setenv ssize 32
setenv sfreqx 1
setenv sfreqy 1
setenv sfreqz 1
if ($#argv >= 4) then
  setenv ssize $4
  setenv sfreqx $5
  setenv sfreqy $6
  setenv sfreqz $7
endif
#
# Problem domain
#
setenv problo " 0.0  1.0 -1.0"
setenv probhi " 1.0  2.0  0.0"
#
# Lists of frequencies
#
setenv freqxall "2.62 4.42 6.22"
setenv freqyall "2.24 4.64 6.44"
setenv freqzall "2.46 4.26 6.66"
#
# Parameters for linear halfspace
#
setenv px 0.43
setenv py 0.52
setenv pz 0.61
setenv nx 1.13
setenv ny 2.22
setenv nz 3.31
#
# Parameters for sphere
#
setenv cx     0.29
setenv cy     0.24
setenv cz     0.19
setenv radius 0.695
#
# Parameters for polar function
#
setenv pcentx  0.43
setenv pcenty  0.52
setenv pcentz  0.57
setenv prad    0.291
setenv ppert  -0.104
setenv pfreq   6
#
# Exterior of a polar function
# Dirichlet domain BC, 1st order Dirichlet EB BC
#
setenv name  polardir1
setenv ebbc  1
setenv dombc 1
setenv geom  2
setenv int   0
runone.csh
#
# Linear halfspace 
# Dirichlet domain BC, 2nd order Dirichlet EB BC
#
setenv name  halfdir2
setenv ebbc  2
setenv dombc 1
setenv geom  0
setenv int   0
runone.csh
#
# Interior of a circle which intersects the domain
# Dirichlet domain BC, 2nd order Dirichlet EB BC
#
setenv name  sphdir2
setenv ebbc  2
setenv dombc 1
setenv geom  1
setenv int   1
runone.csh
#
# Exterior of a polar function
# Dirichlet domain BC, 2nd order Dirichlet EB BC
#
setenv name  polardir2
setenv ebbc  2
setenv dombc 1
setenv geom  2
setenv int   0
runone.csh
#
# Linear halfspace 
# Dirichlet domain BC, Neumann EB BC
#
setenv name  halfneu
setenv ebbc  0
setenv dombc 1
setenv geom  0
setenv int   0
runone.csh
#
# Interior of a circle which intersects the domain
# Dirichlet domain BC, Neumann EB BC
#
setenv name  sphneu
setenv ebbc  0
setenv dombc 1
setenv geom  1
setenv int   1
runone.csh
#
# Exterior of a polar function
# Dirichlet domain BC, Neumann EB BC
#
setenv name  polarneu
setenv ebbc  0
setenv dombc 1
setenv geom  2
setenv int   0
runone.csh
#
# In 3D, publication test case
#
# Problem domain
#
setenv problo "-5.00 -5.00 -5.00"
setenv probhi " 5.00  5.00  5.00"
#
# Lists of frequencies
#
setenv freqxall "1"
setenv freqyall "1"
setenv freqzall "1"
#
# Parameters for sphere
#
setenv cx     5.00
setenv cy     5.00
setenv cz     5.00
setenv radius 4.000
#
# Interior of a circle
# Dirichlet domain BC, 2nd order Dirichlet EB BC
#
setenv name  public
setenv ebbc  2
setenv dombc 1
setenv geom  1
setenv int   1
runone.csh
#
# Generate summaries
#
geterrors.csh $dir
getrates.csh  $dir
foreach name (                 polardir1 \
              halfdir2 sphdir2 polardir2 \
              halfneu  sphneu  polarneu  \
              public                     \
             )
  getsummary.csh $name $dimnum $dir |& tee $dir/summary.$dim.$name
end
