/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef __EBAMRScalarAdvectFactory_H__
#define __EBAMRScalarAdvectFactory_H__

#include "EBAMRScalarAdvect.H"
#include "AMRLevelFactory.H"
#include "EBPatchGodunovFactory.H"
#include "BaseVelocity.H"

/// 
/**
 */
class EBAMRScalarAdvectFactory : public AMRLevelFactory
{
public:
  ///
  AMRLevel* new_amrlevel() const;

  ///
  EBAMRScalarAdvectFactory(const Real&                        a_initialDtMultiplier,
                           const Real&                        a_cfl,
                           const int &                        a_redistRad,
                           const Real&                        a_domainLength,
                           const Real&                        a_refineThresh,
                           const int &                        a_tagBufferSize,
                           const int &                        a_verbosity,
                           const bool&                        a_doSmushing,
                           const BaseVelocityFactory* const   a_velocityFunc,
                           const EBPatchGodunovFactory* const a_patchGodunov);

  ///
  virtual ~EBAMRScalarAdvectFactory();

protected:
  Real                         m_initialDtMultiplier;
  Real                         m_cfl;
  int                          m_redistRad;
  Real                         m_domainLength;
  Real                         m_refineThresh;
  int                          m_tagBufferSize;
  int                          m_verbosity;
  bool                         m_doSmushing;
  const EBPatchGodunovFactory* m_patchGodunov;
  const BaseVelocityFactory*   m_velocityFunc;


private:
  EBAMRScalarAdvectFactory()
  {
    MayDay::Error("invalid operator");
  }

  //disallowed for all the usual reasons
  void operator=(const EBAMRScalarAdvectFactory& a_input)
  {
    MayDay::Error("invalid operator");
  }
  EBAMRScalarAdvectFactory(const EBAMRScalarAdvectFactory& a_input)
  {
    MayDay::Error("invalid operator");
  }
    
};

#endif
