/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _EBMACProjector_H_
#define _EBMACProjector_H_
#include "DisjointBoxLayout.H"
#include "EBISLayout.H"
#include "Box.H"
#include "REAL.H"
#include "LevelData.H"
#include "EBFluxFAB.H"
#include "EBCellFAB.H"
#include "EBSimpleSolver.H"
#include "EBAMRPoissonOp.H"
#include "MultiGrid.H"

/// 
/**
 */
class EBLevelMACProjector
{
public:  

  ///weak construction is bad but often convenient
  EBLevelMACProjector();

  ///
  ~EBLevelMACProjector();

  ///
  void define(const DisjointBoxLayout& a_dbl,
              const EBISLayout&        a_ebisl,
              const Box&               a_domain,
              const Real&              a_dx);

  ///
  /**
     velocity--input and output as divergence-free
     gradient--pure gradient component of input velocity.
   */
  void
  project(LevelData<EBFluxFAB>&  a_velocity,
          LevelData<EBFluxFAB>&  a_gradient);

  void setDefaultValues();

  int computeMGDepth(const DisjointBoxLayout& a_dbl);

  void
  gradient(LevelData<EBFluxFAB>&        a_gradPhi,
           const LevelData<EBCellFAB>&  a_phi);

  void
  gradient(EBFluxFAB&        a_gradPhi,
           const EBCellFAB&  a_phi,
           const EBISBox&    a_ebisBox,
           const Box&        a_box);

  void
  kappaDivergence(LevelData<EBCellFAB>&        a_divVel,
                  const LevelData<EBFluxFAB>&  a_velocity);

  void
  kappaDivergence(EBCellFAB&        a_divVel,
                  const EBFluxFAB&  a_velocity,
                  const EBISBox&    a_ebisBox,
                  const Box&        a_box);
  void 
  solve(LevelData<EBCellFAB>&       a_phi,
        const LevelData<EBCellFAB>& a_rhs);

  //public on purpose
  int m_numSmoothUp;
  int m_numSmoothDown;
  int m_cycles;
  int m_maxIterations;
  Real m_tolerance;

protected:

  DisjointBoxLayout                  m_grids;
  EBISLayout                         m_ebisl;
  Box                                m_domain;
  Real                               m_dx;
  EBSimpleSolver                     m_bottomSolver;
  MultiGrid<LevelData<EBCellFAB> >   m_solver;
  EBAMRPoissonOpFactory              m_levelOpFactory;
  bool                               m_isDefined;
  
private:

  //copy construction and assignment banned
  EBLevelMACProjector(const EBLevelMACProjector& a_input)
  {
    MayDay::Error("We do not allow assignment of objects with pointered data");
  }

  void operator=(const EBLevelMACProjector& a_input)
  {
    MayDay::Error("We do not allow assignment of objects with pointered data");
  }

  

};


#endif
