/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _EBPatchScalarAdvect_H_
#define _EBPatchScalarAdvect_H_


#include "EBPatchGodunov.H"
#include "EBPatchAdvect.H"

///
/**
   An EBPatchGodunov for passive advection with an
   input velocity of a particular function value.
*/
class EBPatchScalarAdvect : public EBPatchAdvect
{
public:

  // Things the app HAS to change
  ///
  /**
   */
  EBPatchScalarAdvect();

  ///
  /**
   */
  virtual ~EBPatchScalarAdvect();

  ///
  /**
     Return number of components for primitive variables.
  */
  virtual int numPrimitives() const
  {
    return 1;
  }

  ///
  /**
     Returns number of components for flux variables.
  */
  virtual int numFluxes() const
  {
    return 1;
  }

  ///
  /**
     Returns number of components for conserved variables.
  */
  virtual int numConserved() const
  {
    return 1;
  }

  ///
  /**
   */
  virtual int numSlopes() const
  {
    return 1;
  }

  ///
  /**
   */
  void
  normalPred(EBCellFAB&       a_primLo,
             EBCellFAB&       a_primHi,
             const EBCellFAB& a_primState,
             const EBCellFAB& a_slopePrim,
             const Real&      a_scale,
             const int&       a_dir,
             const Box&       a_box);

private:

  //disallowed for all the usual reasons
  void operator=(const EBPatchScalarAdvect& a_input)
  {
    MayDay::Error("invalid operator");
  }
  EBPatchScalarAdvect(const EBPatchScalarAdvect& a_input)
  {
    MayDay::Error("invalid operator");
  }
    
};
#endif
