/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental, 
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
#include "EBPatchScalarAdvect.H"
#include "EBPatchScalarAdvectF_F.H"
/*****************************/
void EBPatchScalarAdvect:: 
normalPred(EBCellFAB&       a_rhoLo, 
           EBCellFAB&       a_rhoHi, 
           const EBCellFAB& a_rho, 
           const EBCellFAB& a_dRho, 
           const Real&      a_dtbydx, 
           const int&       a_dir, 
           const Box&       a_box)
{
  assert(m_isVelSet);
  const EBCellFAB& velcc = *m_normalVelPtr;

  BaseFab<Real>& regRhoLo    = a_rhoLo.getSingleValuedFAB();
  BaseFab<Real>& regRhoHi    = a_rhoHi.getSingleValuedFAB();
  const BaseFab<Real>& regVel =  velcc.getSingleValuedFAB();
  const BaseFab<Real>& regRho =  a_rho.getSingleValuedFAB();
  const BaseFab<Real>& regDRho= a_dRho.getSingleValuedFAB();
  
  FORT_PREDADVECT(CHF_BOX(a_box),
                  CHF_CONST_FRA1(regRho,0),
                  CHF_CONST_FRA1(regDRho, 0),
                  CHF_CONST_FRA(regVel),
                  CHF_FRA1(regRhoLo, 0),
                  CHF_FRA1(regRhoHi, 0),
                  CHF_CONST_INT(a_dir),
                  CHF_CONST_REAL(a_dtbydx));

  IntVectSet ivsMulti = m_ebisBox.getMultiCells(a_box);
  for(VoFIterator vofit(ivsMulti, m_ebisBox.getEBGraph()); vofit.ok(); ++vofit)
    {
      const VolIndex& vof = vofit();
      Real dense, denlo, denhi, denslope;
      RealVect veloc;

      dense    = a_rho(vof, 0);
      denslope = a_dRho(vof, 0);
      for(int idir = 0; idir < SpaceDim; idir++)
        {
          veloc[idir] = velcc(vof, idir);
        }

      FORT_POINTPREDADVECT(CHF_REAL(dense),
                           CHF_REAL(denlo),
                           CHF_REAL(denhi),
                           CHF_REAL(denslope),
                           CHF_REALVECT(veloc),
                           CHF_CONST_INT(a_dir),
                           CHF_CONST_REAL(a_dtbydx));

      a_rhoLo(vof, 0) = denlo;
      a_rhoHi(vof, 0) = denhi;
    }
}
/*****************************/
EBPatchScalarAdvect::
EBPatchScalarAdvect():EBPatchAdvect()
{
}
/*****************************/
EBPatchScalarAdvect::
~EBPatchScalarAdvect()
{
}
