/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "RotVel.H"
#include "MayDay.H"
#include "PolyGeom.H"
/*****/
RealVect SolidRotVel::
getVel(const RealVect& a_xloc)
{
  RealVect radVec = a_xloc - m_center;
  RealVect retval;
  if(CH_SPACEDIM==3)
    {
      RealVect crossVec = PolyGeom::cross(radVec, m_axis);
      crossVec *= m_constant;
      retval = crossVec;
    }
  else
    {
      retval[0] = -radVec[1];
      retval[1] =  radVec[0];
      retval *= m_constant;
    }
  return retval;
}
/*****/
SolidRotVel::
SolidRotVel(const RealVect& a_center, const RealVect& a_axis, const Real& a_constant)
{
  m_center   = a_center;
  m_axis     = a_axis;
  Real dot;
  PolyGeom::unifyVector(m_axis, dot);
  m_constant = a_constant;
}
/*****/
BaseVelocity* SolidRotVelFactory::
create() const
{
  SolidRotVel* retval = new SolidRotVel(m_center, m_axis, m_constant);
  return (BaseVelocity *)retval;  
}
/****/
SolidRotVelFactory:: 
SolidRotVelFactory(const RealVect& a_center, const RealVect& a_axis, const Real& a_constant)
{
  m_center   = a_center;
  m_axis     = a_axis;
  Real dot;
  PolyGeom::unifyVector(m_axis, dot);
  m_constant = a_constant;
}
/****/
