/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _AMR_LEVEL_IDEAL_MHD_FACTORY_H_
#define _AMR_LEVEL_IDEAL_MHD_FACTORY_H_

#include "AMRLevelFactory.H"
#include "AMRLevelIdealMHD.H"

/// AMR Godunov factory
/**
 */
class AMRLevelIdealMHDFactory : public AMRLevelFactory
{
public:
  /// Null constructor
  /**
   */
  AMRLevelIdealMHDFactory();

  /// Virtual constructor
  /**
   */
  virtual AMRLevel* new_amrlevel() const;

  /// Destructor
  /**
   */
  virtual ~AMRLevelIdealMHDFactory();

  /// CFL number
  /**
   */
  virtual void CFL(Real a_cfl);

  /// Physical dimension of the longest side of the domain
  /**
   */
  virtual void domainLength(Real a_domainLength);

///
  /**
   */

  virtual void verbosity(const int& verbosity);

  /// Refinement threshold
  /**
   */
  virtual void refinementThreshold(Real a_refineThresh);

  /// Tag buffer size
  /**
   */
  void tagBufferSize(int a_tagBufferSize);

  /// Initial dt multiplier
  /**
   */
  void initialDtMultiplier(Real a_initialDtMultiplier);

  /// PatchGodunov object (used as a factory)
  /**
   */
  virtual void patchGodunov(RefCountedPtr<PatchGodunov> a_patchGodunov);

  /// PoissonOp object (used as a factory)
  /**
   */
  virtual void poissonOp(RefCountedPtr<PoissonOp> a_poissonOp);

  /// Check that everything is defined
  /**
   */
  bool isDefined() const;

protected:
  // Some default values
  void setDefaultValues();

  int m_verbosity;

  // CFL number
  Real m_cfl;
  bool m_cflSet;

  // Physical dimension of the longest side of the domain
  Real m_domainLength;
  bool m_domainLengthSet;

  // Refinement threshold for gradient
  Real m_refineThresh;
  bool m_refineThreshSet;

  // Tag buffer size
  int  m_tagBufferSize;
  bool m_tagBufferSizeSet;

  // Initial dt multiplier 
  Real m_initialDtMultiplier;
  bool m_initialDtMultiplierSet;

  // Patch integrator (used as a factory)
  RefCountedPtr<PatchGodunov> m_patchGodunov;
  bool                        m_patchGodunovSet;

  RefCountedPtr<PoissonOp> m_poissonOp;
  bool                        m_poissonOpSet;

private:
  // Disallowed for all the usual reasons
  void operator=(const AMRLevelIdealMHDFactory& a_input)
  {
    MayDay::Error("invalid operator");
  }

  // Disallowed for all the usual reasons
  AMRLevelIdealMHDFactory(const AMRLevelIdealMHDFactory& a_input)
  {
    MayDay::Error("invalid operator");
  }
};

#endif
