/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DFMartin, Sun, May 5, 2002

#include "GSRBSmoother.H"
#include "LayoutIterator.H"
#include "BoxIterator.H"

GSRBSmoother::GSRBSmoother()
{
  // set default value for m_numSmooth
  m_numSmooth = 60;
}

GSRBSmoother::~GSRBSmoother()
{
}

BaseBottomSmoother* GSRBSmoother::new_bottomSmoother() const
{
  GSRBSmoother* newsmoother = new GSRBSmoother();

  if (newsmoother == NULL)
    {
      MayDay::Error("Out of Memory in GSRBSmoother::new_bottomSmoother");
    }

  return static_cast<BaseBottomSmoother*>(newsmoother);
}

/***********************/
// True to its name, we do a bunch of GSRB here
/***********************/
void GSRBSmoother::doBottomSmooth(LevelData<FArrayBox>&       a_phi,
                                  const LevelData<FArrayBox>& a_rhs,
                                  LevelOp*                    a_levelopPtr)
{

  for (int n = 0; n < m_numSmooth; n++)
    {
      a_levelopPtr->smooth(a_phi, a_rhs);
    }
}

void GSRBSmoother::setNumSmooth(int a_numSmooth)
{
  m_numSmooth = a_numSmooth;
}

int GSRBSmoother::numSmooth() const
{
  return m_numSmooth;
}
