/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DTGraves, Weds, July 21, 1999

#include "DotProdF_F.H"
#include "DotProduct.H"
#include "SPMD.H"

Real 
DotProduct(const BoxLayoutData<FArrayBox>& a_dataOne, 
           const BoxLayoutData<FArrayBox>& a_dataTwo,
           const BoxLayout& a_dblIn)
{
  Interval comps = a_dataOne.interval();
  return DotProduct(a_dataOne, a_dataTwo, a_dblIn, comps);
}


extern Real
DotProduct(const BoxLayoutData<FArrayBox>& a_dataOne, 
           const BoxLayoutData<FArrayBox>& a_dataTwo,
           const BoxLayout& a_dblIn,
           const Interval& a_comps)
{
  Vector<Real> rhodot;
  rhodot.reserve(10);
  const int startcomp = a_comps.begin();
  const int endcomp = a_comps.end();
  //calculate the single-processor dot product
  DataIterator dit = a_dataOne.dataIterator();
  
  for(dit.reset(); dit.ok(); ++dit)
    {
      Box fabbox = a_dblIn.get(dit());
      const FArrayBox& onefab = a_dataOne[dit()];
      const FArrayBox& twofab = a_dataTwo[dit()];
      assert(onefab.box().contains(fabbox));
      assert(twofab.box().contains(fabbox));

      Real dotgrid = 0;
      FORT_DOTPRODUCT(CHF_REAL(dotgrid),
                      CHF_CONST_FRA(onefab), 
                      CHF_CONST_FRA(twofab),
                      CHF_BOX(fabbox),
                      CHF_CONST_INT(startcomp),
                      CHF_CONST_INT(endcomp));
      
      rhodot.push_back(dotgrid);
    }

  // now for the multi-processor fandango

  //gather all the rhodots onto a vector and add them up
  int baseProc = 0;
  Vector<Vector<Real> >dotVec;
  gather(dotVec, rhodot, baseProc);

  Real rhodotTot = 0.0;
  if(procID() == baseProc)
    {
	  assert(dotVec.size() == numProc());
	  rhodot.resize(a_dblIn.size());
	  int index=0;
	  for(int p=0; p<dotVec.size(); p++)
		{
		  Vector<Real>& v = dotVec[p];
		  for(int i=0; i< v.size(); i++, index++)
			rhodot[index]=v[i];
		}
	  rhodot.sort();
      for(int ivec = 0; ivec < rhodot.size(); ivec++)
        {
          rhodotTot += rhodot[ivec];
        }
    }
  //broadcast the sum to all processors.
  broadcast(rhodotTot, baseProc);

  //return the total
  return rhodotTot;
}

