/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _DIRICHLETDOMAINBC_H_
#define _DIRICHLETDOMAINBC_H_

#include "RefCountedPtr.H"

#include "BaseDomainBC.H"
#include "BaseBCValue.H"

class DirichletDomainBC: public BaseDomainBC
{
public:
  DirichletDomainBC();

  virtual ~DirichletDomainBC();

  virtual void setValue(Real a_value);

  virtual void setFunction(RefCountedPtr<BaseBCValue> a_func);

  virtual void setEBOrder(int a_ebOrder);

  virtual void getFaceFlux(FArrayBox&       a_faceFlux,
                           const FArrayBox& a_phi,
                           const RealVect&  a_probLo,
                           const RealVect&  a_dx,
                           int              a_idir,
                           Side::LoHiSide   a_side,
                           bool             a_useHomogeneous);

  virtual void getFaceFlux(Real&            a_faceFlux,
                           const VolIndex&  a_vof,
                           const EBCellFAB& a_phi,
                           const RealVect&  a_probLo,
                           const RealVect&  a_dx,
                           int              a_idir,
                           Side::LoHiSide   a_side,
                           bool             a_useHomogeneous);

private:
  bool m_onlyHomogeneous;
  bool m_isFunctional;

  Real m_value;
  RefCountedPtr<BaseBCValue> m_func;

  int m_ebOrder;
};

#endif
