/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _DIRICHLETEBBC_H_
#define _DIRICHLETEBBC_H_

#include "RefCountedPtr.H"

#include "BaseEBBC.H"
#include "BaseBCValue.H"

class DirichletEBBC: public BaseEBBC
{
public:
  DirichletEBBC(const ProblemDomain& a_domain,
                const EBISLayout&    a_layout,
                const RealVect&      a_dx);

  virtual ~DirichletEBBC();

  virtual void define();

  virtual void setOrder(int a_order);

  virtual void setValue(Real a_value);

  virtual void setFunction(RefCountedPtr<BaseBCValue> a_func);

  virtual void getEBFlux(Real&                       a_flux,
                         const VolIndex&             a_vof,
                         const LevelData<EBCellFAB>& a_phi,
                         const DataIndex             a_dit,
                         const RealVect&             a_probLo,
                         const RealVect&             a_dx,
                         bool                        a_useHomogeneous);

private:
  virtual void getFirstOrderStencil(VoFStencil&     a_stencil,
                                    Real&           a_weight,
                                    const VolIndex& a_vof,
                                    const EBISBox&  a_ebisBox, 
                                    const RealVect& a_dx);

  virtual void getSecondOrderStencil(VoFStencil&     a_stencil,
                                     Real&           a_weight,
                                     const VolIndex& a_vof,
                                     const EBISBox&  a_ebisBox, 
                                     const RealVect& a_dx);

  virtual void johanDropOrder(bool&          a_droppedOrder,
                              const IntVect  a_interpCen[2],
                              const int      a_planeDir[SpaceDim-1],
                              const EBISBox& a_ebisBox);

  VoFStencil BuildEQIStencil(const VolIndex a_vofs[3],
                             const Real&    a_distance);

  VoFStencil BuildEQIStencil(VoFStencil a_vofs[3],
                             const Real& a_distance);

  VoFStencil ivofStencil(const VoFStencil a_iPlaneStencil[2],
                         const Real&      a_distance1,
                         const Real&      a_distance2);

  bool m_isDefined;

  bool m_onlyHomogeneous;
  bool m_isFunction;

  int m_order;

  Real m_value;
  RefCountedPtr<BaseBCValue> m_func;

  ProblemDomain m_domain;
  EBISLayout    m_layout;

  RealVect m_dx;

  LayoutData<BaseIVFAB<Real> >       m_fluxWeight;
  LayoutData<BaseIVFAB<VoFStencil> > m_fluxStencil;
};

class DirichletEBBCFactory: public BaseEBBCFactory
{
public:
  DirichletEBBCFactory();

  virtual ~DirichletEBBCFactory();

  virtual void setOrder(int a_order);

  virtual void setValue(Real a_value);

  virtual void setFunction(RefCountedPtr<BaseBCValue> a_func);

  virtual DirichletEBBC* newEBBC(const ProblemDomain& a_domain,
                                 const EBISLayout&    a_layout,
                                 const RealVect&      a_dx);

private:
  bool m_onlyHomogeneous;
  bool m_isFunction;

  int m_order;

  Real m_value;
  RefCountedPtr<BaseBCValue> m_func;
};

#endif
