/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _EBLEVELDATAOPS_H_
#define _EBLEVELDATAOPS_H_

#include "IntVect.H"
#include "LevelData.H"
#include "RefCountedPtr.H"
#include "LevelDataOps.H"

#include "EBCellFAB.H"

#define EBLEVELDATAOPS_INTERIORREGVOFS   0x1
#define EBLEVELDATAOPS_BOUNDARYREGVOFS   0x2
#define EBLEVELDATAOPS_INTERIORIRREGVOFS 0x4
#define EBLEVELDATAOPS_BOUNDARYIRREGVOFS 0x8

#define EBLEVELDATAOPS_ALLVOFS         (EBLEVELDATAOPS_INTERIORREGVOFS   | \
                                        EBLEVELDATAOPS_BOUNDARYREGVOFS   | \
                                        EBLEVELDATAOPS_INTERIORIRREGVOFS | \
                                        EBLEVELDATAOPS_BOUNDARYIRREGVOFS )

#define EBLEVELDATAOPS_REGULARVOFS     (EBLEVELDATAOPS_INTERIORREGVOFS | \
                                        EBLEVELDATAOPS_BOUNDARYREGVOFS )

#define EBLEVELDATAOPS_IRREGULARVOFS   (EBLEVELDATAOPS_INTERIORIRREGVOFS | \
                                        EBLEVELDATAOPS_BOUNDARYIRREGVOFS )

#define EBLEVELDATAOPS_BOUNDARYVOFS    (EBLEVELDATAOPS_BOUNDARYREGVOFS | \
                                        EBLEVELDATAOPS_IRREGULARVOFS   )

class EBLevelDataOps: public LevelDataOps<EBCellFAB>
{
public:
  EBLevelDataOps()
    :m_ebLevelFactory(NULL)
  {
    m_ebLevelFactory = new DefaultDataFactory<EBCellFAB>();

    m_noParity  = IntVect::TheZeroVector();
    m_noParity -= IntVect::TheUnitVector();
  }

  EBLevelDataOps(RefCountedPtr<DataFactory<EBCellFAB> > a_factoryPtr)
    :m_ebLevelFactory(a_factoryPtr)
  {
  }

  virtual ~EBLevelDataOps(){;}

  virtual void setVal(LevelData<EBCellFAB>& a_result,
                      Real                  a_value);

  virtual void sum(LevelData<EBCellFAB>&       a_result,
                   const LevelData<EBCellFAB>& a_in1,
                   const LevelData<EBCellFAB>& a_in2);

  virtual void sum(LevelData<EBCellFAB>&       a_result,
                   const LevelData<EBCellFAB>& a_in1,
                   const LevelData<EBCellFAB>& a_in2,
                   const IntVect&              a_parity);

  virtual void addConstant(LevelData<EBCellFAB>& a_data,
                           Real                  a_constant);

  virtual void product(LevelData<EBCellFAB>&       a_result,
                       const LevelData<EBCellFAB>& a_in1,
                       const LevelData<EBCellFAB>& a_in2);

  virtual void product(LevelData<EBCellFAB>&       a_result,
                       const LevelData<EBCellFAB>& a_in1,
                       const LevelData<EBCellFAB>& a_in2,
                       const IntVect&              a_parity);

  virtual void kappaWeight(LevelData<EBCellFAB>& a_data);

  virtual Real kappaNorm(Real&                       a_volume,
                         const LevelData<EBCellFAB>& a_data,
                         int                         a_which,
                         const ProblemDomain&        a_domain,
                         int                         a_p=2);

  virtual Real noKappaNorm(Real&                       a_volume,
                           const LevelData<EBCellFAB>& a_data,
                           int                         a_which,
                           const ProblemDomain&        a_domain,
                           int                         a_p=2);

  virtual Real kappaDotProduct(Real&                       a_volume,
                               const LevelData<EBCellFAB>& a_data1,
                               const LevelData<EBCellFAB>& a_data2,
                               int                         a_which,
                               const ProblemDomain&        a_domain);

  virtual Real noKappaDotProduct(Real&                       a_volume,
                                 const LevelData<EBCellFAB>& a_data1,
                                 const LevelData<EBCellFAB>& a_data2,
                                 int                         a_which,
                                 const ProblemDomain&        a_domain);

  virtual Real kappaSumLevel(Real&                       a_volume,
                             const LevelData<EBCellFAB>& a_data,
                             int                         a_which,
                             const ProblemDomain&        a_domain);

  virtual Real noKappaSumLevel(Real&                       a_volume,
                               const LevelData<EBCellFAB>& a_data,
                               int                         a_which,
                               const ProblemDomain&        a_domain);

protected:
  RefCountedPtr<DataFactory<EBCellFAB> > m_ebLevelFactory;

  IntVect m_noParity;

  virtual Real sumKappaPow(Real&                a_volume,
                           const EBCellFAB&     a_data,
                           int                  a_which,
                           const ProblemDomain& a_domain,
                           int                  a_p);

  virtual Real sumNoKappaPow(Real&                a_volume,
                             const EBCellFAB&     a_data,
                             int                  a_which,
                             const ProblemDomain& a_domain,
                             int                  a_p);

  virtual Real sumKappaDotProduct(Real&                a_volume,
                                  const EBCellFAB&     a_data1,
                                  const EBCellFAB&     a_data2,
                                  int                  a_which,
                                  const ProblemDomain& a_domain);

  virtual Real sumNoKappaDotProduct(Real&                a_volume,
                                    const EBCellFAB&     a_data1,
                                    const EBCellFAB&     a_data2,
                                    int                  a_which,
                                    const ProblemDomain& a_domain);

  virtual Real sumKappa(Real&                a_volume,
                        const EBCellFAB&     a_data,
                        int                  a_which,
                        const ProblemDomain& a_domain);

  virtual Real sumNoKappa(Real&                a_volume,
                          const EBCellFAB&     a_data,
                          int                  a_which,
                          const ProblemDomain& a_domain);
};
#endif
