/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _NEUMANNEBBC_H_
#define _NEUMANNEBBC_H_

#include "RefCountedPtr.H"

#include "BaseEBBC.H"
#include "BaseBCFlux.H"

class NeumannEBBC: public BaseEBBC
{
public:
  NeumannEBBC(const ProblemDomain& a_domain,
              const EBISLayout&    a_layout,
              const RealVect&      a_dx);

  virtual ~NeumannEBBC();

  virtual void setValue(Real a_value);

  virtual void setFunction(RefCountedPtr<BaseBCFlux> a_flux);

  virtual void getEBFlux(Real&                       a_flux,
                         const VolIndex&             a_vof,
                         const LevelData<EBCellFAB>& a_phi,
                         const DataIndex             a_dit,
                         const RealVect&             a_probLo,
                         const RealVect&             a_dx,
                         bool                        a_useHomogeneous);

private:
  bool m_onlyHomogeneous;
  bool m_isFunction;

  Real m_value;
  RefCountedPtr<BaseBCFlux> m_flux;
};

class NeumannEBBCFactory: public BaseEBBCFactory
{
public:
  NeumannEBBCFactory();

  virtual ~NeumannEBBCFactory();

  virtual void setValue(Real a_value);

  virtual void setFunction(RefCountedPtr<BaseBCFlux> a_flux);

  virtual NeumannEBBC* newEBBC(const ProblemDomain& a_domain,
                               const EBISLayout&    a_layout,
                               const RealVect&      a_dx);

private:
  bool m_onlyHomogeneous;
  bool m_isFunction;

  Real m_value;
  RefCountedPtr<BaseBCFlux> m_flux;
};

#endif
