/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _EBPatchAdvect_H_
#define _EBPatchAdvect_H_


#include "EBPatchGodunov.H"

///
/**
   An EBPatchGodunov for passive advection with an
   input velocity of a particular function value.
*/
class EBPatchAdvect : public EBPatchGodunov
{
public:

  // Things the app HAS to change
  ///
  /**
   */
  EBPatchAdvect();

  ///
  /**
   */
  virtual ~EBPatchAdvect();

  ///
  /**
     Return number of components for primitive variables.
  */
  virtual int numPrimitives() const = 0;

  ///
  /**
     Returns number of components for flux variables.
  */
  virtual int numFluxes() const = 0;

  ///
  /**
     Returns number of components for conserved variables.
  */
  virtual int numConserved() const = 0;

  ///
  /**
   */
  virtual int numSlopes() const = 0;


  ///
  /**
   */
  virtual void
  normalPred(EBCellFAB&       a_primLo,
             EBCellFAB&       a_primHi,
             const EBCellFAB& a_primState,
             const EBCellFAB& a_slopePrim,
             const Real&      a_scale,
             const int&       a_dir,
             const Box&       a_box) = 0;


  // Things The App Might Change

  ///
  /**
     default does -= u_idir d cons/ dx_idir.
     assumes fluxes are really prims.
  */
  virtual void 
  updatePrim(EBCellFAB&              a_primMinu, 
             EBCellFAB&              a_primPlus, 
             const EBFaceFAB&        a_flux, 
             const BaseIVFAB<Real>&  a_coveredFluxMinu, 
             const BaseIVFAB<Real>&  a_coveredFluxPlus, 
             const Vector<VolIndex>& a_coveredFaceMinu, 
             const Vector<VolIndex>& a_coveredFacePlus, 
             const int&              a_dir, 
             const Box&              a_box, 
             const Real&             a_scale);


  ///default noop
  virtual void 
  floorPrimitives(EBCellFAB&  a_primState,
                  const Box&  a_box)
  {;}

  ///default noop
  virtual void 
  floorConserved(EBCellFAB&  a_consState,
                 const Box&  a_box)
  {;}

  ///default noop
  virtual void 
  floorConserved(BaseIVFAB<Real>&  a_consState,
                 const IntVectSet&  a_set)
  {;}

  ///default noop
  virtual void 
  floorPrimitives(BaseIVFAB<Real>&  a_primState,
                  const IntVectSet&  a_set)
  {;}

  /// default noop
  virtual void  getCoveredValuesPrim(Vector<Real>& a_covValues)
  {;}

  ///default noop
  virtual void  getCoveredValuesCons(Vector<Real>& a_covValues)
  {;}

  ///default noop
  virtual void setCoveredConsVals(EBCellFAB& a_consState)
  {;}


  ///
  /**
     just a copy as default
  */
  virtual void 
  consToPrim(EBCellFAB&       a_primState,
             const EBCellFAB& a_conState,
             const Box&       a_box,
             int              a_logflag);

  ///
  /**
     just a copy as default
   */
  virtual void 
  consToPrim(BaseIVFAB<Real>&       a_primState,
             const BaseIVFAB<Real>& a_conState,
             const IntVectSet&      a_ivs);
  
  ///
  /**
     just a copy
   */
  virtual void 
  consToPrim(BaseIVFAB<Real>&  a_primState,
             const EBCellFAB&  a_conState,
             const IntVectSet& a_ivs);
  

  ///
  /**
     just a copy
   */
  virtual void 
  primToCons(EBCellFAB&       a_primState,
             const EBCellFAB& a_conState,
             const Box&       a_box);
  
  ///
  /**
     just a copy
   */
  virtual void 
  primToCons(BaseIVFAB<Real>&       a_primState,
             const BaseIVFAB<Real>& a_conState,
             const IntVectSet&      a_ivs);

  ///
  /**
     sends back primitive state (NOT FLUX) using upwinding
  */
  virtual void 
  riemann(EBFaceFAB&       a_primState,
          const EBCellFAB& a_primLeft,
          const EBCellFAB& a_primRight,
          const int&       a_dir,
          const Box&       a_box);

  ///
  /**
     sends back primitive state (NOT FLUX) using upwinding
  */
  virtual void 
  riemann(BaseIVFAB<Real>&        a_coveredFlux,
          const BaseIVFAB<Real>&  a_extendedState,
          const EBCellFAB&        a_primState,
          const Vector<VolIndex>& a_region,
          const int&              a_dir,
          const Side::LoHiSide&   a_sd);

  ///
  /**
     sets eb flux to zero
   */
  virtual void
  computeEBIrregFlux(BaseIVFAB<Real>&  a_ebIrregFlux,
                     const EBCellFAB&  a_primState,
                     const EBCellFAB&  a_nonConsDivF,
                     const EBCellFAB   a_slopePrim[SpaceDim],
                     const IntVectSet& a_irregIVS,
                     const EBCellFAB&  a_flux);


  ///
  /**
   */
  void 
  extrapolatePrim(EBFluxFAB&                       a_flux, 
                  EBCellFAB&                       a_primState,
                  EBCellFAB                        a_slopePrim[SpaceDim],
                  EBCellFAB                        a_slopeNLim[SpaceDim],
                  Vector<BaseIVFAB<Real> * >&      a_coveredFluxMinu, 
                  Vector<BaseIVFAB<Real> * >&      a_coveredFluxPlus, 
                  const Vector<IntVectSet >&       a_coveredSetsMinu, 
                  const Vector<IntVectSet >&       a_coveredSetsPlus, 
                  const Vector<Vector<VolIndex> >& a_coveredFaceMinu, 
                  const Vector<Vector<VolIndex> >& a_coveredFacePlus, 
                  const EBCellFAB&                 a_flattening,
                  const EBCellFAB&                 a_consState, 
                  const EBCellFAB&                 a_source, 
                  const Box&                       a_box,
                  bool                             a_verbose);

  ///
  virtual void
  advectiveDerivative(EBCellFAB                      & a_udotDelRho,
                      const EBFluxFAB                & a_faceRho,
                      const EBFluxFAB                & a_faceVel,
                      const Vector<BaseIVFAB<Real>*> & a_coveredRhoMinu,
                      const Vector<BaseIVFAB<Real>*> & a_coveredRhoPlus,
                      const Vector<BaseIVFAB<Real>*> & a_coveredVelMinu,
                      const Vector<BaseIVFAB<Real>*> & a_coveredVelPlus,
                      const Vector<Vector<VolIndex> >& a_coveredFaceMinu,
                      const Vector<Vector<VolIndex> >& a_coveredFacePlus,
                      const Box                      & a_box);

  //Things you do not want to mess with unless you really know what you are doing
  ///
  /**
     always false for now
  */
  virtual bool usesFlattening() const;
  
  ///
  /**
     always false for now
  */
  virtual bool usesArtificialViscosity() const;
  
  ///
  /**
     Return true if you are using fourth-order slopes.
     Return false if you are using second-order slopes.
  */
  virtual bool usesFourthOrderSlopes() const;

  ///
  void
  averageVelToCC(EBCellFAB&                        a_normalVel, 
                 const EBFluxFAB&                  a_advectionVel,
                 const Vector<BaseIVFAB<Real> * >& a_coveredVeloLo, 
                 const Vector<BaseIVFAB<Real> * >& a_coveredVeloHi, 
                 const Vector<Vector<VolIndex> >&  a_coveredFaceLo, 
                 const Vector<Vector<VolIndex> >&  a_coveredFaceHi, 
                 const Box&                        a_box) const;

  virtual void setVelocities(const EBCellFAB& a_normalVel,
                             const EBFluxFAB& a_advectionVel);


  virtual void 
  setValidBox(const Box& a_validBox,
              const EBISBox& a_ebisBox,
              const IntVectSet& a_coarseFineIVS,
              const Real& a_time,
              const Real& a_dt);

  ///sets generic defaults 
  virtual Vector<string> stateNames();
  ///sets generic defaults
  virtual Vector<string> primNames();

  ///
  void useLimiting(bool a_limiting);

  // Things that should not be called because they do not apply

  /// should not be called
  Real 
  getMaxWaveSpeed(const EBCellFAB& a_consState,
                  const Box& a_box)
  {
    MayDay::Error("should not be called ");
    return -1.0;
  }
  /// should not be called
  void 
  getFlux(EBFaceFAB&       a_flux,
          const EBFaceFAB& a_prim,
          const int&       a_dir,
          const Box&       a_box)
  {
    MayDay::Error("should not be called ");
  }

  ///
  /**
     should not be called
  */
  Interval velocityInterval() const
  {
    MayDay::Error("should not be called");
    Interval retval;
    return retval;
  }

  ///
  /**
     should not be called
  */
  virtual int densityIndex() const
  {
    return 0;
  }

  ///     should not be called
  int pressureIndex() const
  {
    MayDay::Error("should not be called");
    return -1;
  }

  ///     should not be called
  int bulkModulusIndex() const
  {
    MayDay::Error("should not be called");
    return -1;
  }
  
  ///     should not be called
  Real artificialViscosityCoefficient() const
  {
    MayDay::Error("should not be called");
    return -1.0;
  }

protected: 

  const EBFluxFAB* m_advectionVelPtr;
  const EBCellFAB* m_normalVelPtr;
  bool m_isVelSet;

private:
  //disallowed for all the usual reasons
  void operator=(const EBPatchAdvect& a_input)
  {
    MayDay::Error("invalid operator");
  }
  EBPatchAdvect(const EBPatchAdvect& a_input)
  {
    MayDay::Error("invalid operator");
  }
    
};
#endif
