/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//


#ifndef _EBPHYSIBC_H
#define _EBPHYSIBC_H

#include "EBCellFAB.H"
#include "EBISLayout.H"
#include "EBFaceFAB.H"
#include "REAL.H"
#include "LevelData.H"
#include "ProblemDomain.H"

///
/**
   Pure virtual base class through which a user specifies the
   initial and boundary conditions for LevelGodunov.
 */
class EBPhysIBC
{
public:
  ///
  EBPhysIBC();

  ///
  virtual ~EBPhysIBC();

  ///
  virtual void define(const ProblemDomain&  a_domain,
                      const Real& a_dx) = 0;

  ///  For every box in this level, this virtual function is called.
  virtual void fluxBC(EBFaceFAB&            a_flux,
                      const EBCellFAB&      a_Wcenter,
                      const EBCellFAB&      a_Wextrap,
                      const Side::LoHiSide& a_sd,
                      const Real&           a_time,
                      const EBISBox&   a_ebisBox,
                      const Box&       a_box,
                      const int&       a_dir) = 0;

  /// Put initial conserved state into a_conState
  virtual void initialize(LevelData<EBCellFAB>& a_conState,
                          const EBISLayout& a_ebisl) const = 0;
  

  /// Set boundary slopes
  /**
     The boundary slopes in a_dW are already set to one sided difference
     approximations.  If this function doesn't change them they will be
     used for the slopes at the boundaries.
   */
  virtual void setBndrySlopes(EBCellFAB&       a_deltaPrim,
                              const EBCellFAB& a_primState,
                              const EBISBox&   a_ebisBox,
                              const Box&       a_box,
                              const int&       a_dir) = 0;

};

#endif
