/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DTGraves, Thurs, Aug 16, 2001

#ifndef _EBAMRIO_
#define _EBAMRIO_
#include "LevelData.H"
#include "REAL.H"
#include "Vector.H"
#include "EBCellFAB.H"
#include "IntVectSet.H"
#include "EBISBox.H"
#include "EBISLayout.H"
#include "EBCellFAB.H"
#include "EBFaceFAB.H"
#include <string>
using std::string;
using std::istream;

#ifdef HDF5
///
/**
   Writes hierarchy of levels in HDF5 format.  Only available if the
   preprocessor macro HDF5 is defined at compilation.  Writes
   single-valued data using regular Chombo WriteAMRHierarchyHDF5
   
   {\bf Arguments:}\\
   a_filename :  file to output to.\\
   a_vectGrids : grids at each level.\\
   a_vectData :  data at each level.\\
   a_vectNames:  names of variables.\\
   a_domain :  domain at coarsest level.\\
   a_dx     :  grid spacing at coarsest level.\\
   a_dt     :  time step at coarsest level.\\
   a_time     :  time.\\
   a_vectRatio :  refinement ratio at all level.\\
   a_numLevels :  number of levels to output.\\ 
   a_replaceCovered:  flag for replacing covered values with a_coveredVal.\\
   a_coveredVal:  sets Covered values to these if a_replaceCovered is true.\\
   (ith entry is refinement ratio between levels i and i + 1).\\
   This is blocking.
*/
void
writeEBHDF5(const string& filename,
            const Vector<DisjointBoxLayout>& a_vectGrids,
            const Vector<LevelData<EBCellFAB>* > & a_vectData,
            const Vector<string>& a_vectNames,
            const Box& a_domain,
            const Real& a_dx,
            const Real& a_dt,
            const Real& a_time,
            const Vector<int>& a_vectRatio,
            const int& a_numLevels,
            const bool& a_replaceCovered,
            const Vector<Real>& a_coveredValues);

///
/**
 */
void
writeEBHDF5(const string& a_filename,
            const DisjointBoxLayout& a_grids,
            const LevelData<EBCellFAB>& a_data,
            const Vector<string>& a_vectNames,
            const Box& a_domain,
            const Real& a_dx,
            const Real& a_dt,
            const Real& a_time,
            const bool& a_replaceCovered,
            const Vector<Real>& a_coveredValues);

///
/**
 */
void
writeEBHDF5(const string& a_filename,
            const Box& a_grid,
            const EBCellFAB& a_data,
            const Vector<string>& a_vectNames,
            const Box& a_domain,
            const Real& a_dx,
            const Real& a_dt,
            const Real& a_time,
            const bool& a_replaceCovered,
            const Vector<Real>& a_coveredValues);

/// 
/** Writes a plotfile using the same format as writeEBHDF5, but
    for a EBCellFAB.  This is useful for debugging.  *a_dataPtr is written 
    to a file named fab.hdf5
*/
void
writeEBFAB(const EBCellFAB* a_dataPtr);

/// 
/** View *a_dataPtr by writing it to an HDF5 plotfile (to a temporary file)
    and then running ChomboVis with a python script which brings up a data
    browser by default. The file has the same format as writeEBHDF5,
    but for a single EBCellFAB.  This is useful for debugging.
*/
void
viewEBFAB(const EBCellFAB* a_dataPtr);

///
/** Writes a plotfile using the same format as writeEBHDF5, but
    for a EBCellFAB.  This is useful for debugging.  *a_dataPtr is written
    to the file given by a_filename.
*/
void
writeEBFABname(const EBCellFAB* a_dataPtr,
               const char*      a_filename);

/// 
/** View the faced centered, regular data (as cell center data) in *a_dataPtr
    by writing it to an HDF5 plotfile (to a temporary file) and then running
    ChomboVis with a python script which brings up a data browser by default.
    The file has the same format as writeEBHDF5, but for a single EBFaceFAB.
    This is useful for debugging.
*/
void
viewEBFace(const EBFaceFAB* a_dataPtr);

///
/** Write a plotfile using the same format as writeEBHDF5, but
    for a single LevelData<EBCellFAB>.  Useful for debugging.  *a_dataPtr is
    written to a file named LDF.hdf5.
*/
void
writeEBLevel(const LevelData<EBCellFAB>* a_dataPtr);

/// 
/** View *a_dataPtr by writing it to an HDF5 plotfile (to a temporary file)
    and then running ChomboVis with a python script which brings up a data
    browser by default. The file has the same format as writeEBHDF5,
    but for a single LevelData<EBCellFAB>.  This is useful for debugging.
*/
void
viewEBLevel(const LevelData<EBCellFAB>* a_dataPtr);

///
/** Write a plotfile using the same format as writeEBHDF5, but
    for a single LevelData<EBCellFAB>. Useful for debugging.  *a_dataPtr is
    written to the file given by a_filename.
*/
void
writeEBLevelname(const LevelData<EBCellFAB>* a_dataPtr,
                 const char*                 a_filename);

#endif  //HDF5

#endif
