/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// dtgraves mon oct 15 2001

#ifndef _eblevel_redist_stencil_H_
#define _eblevel_redist_stencil_H_

#include "REAL.H"
#include "FArrayBox.H"
#include "LevelData.H"
#include "DisjointBoxLayout.H"
#include "EBISLayout.H"
#include "EBCellFAB.H"
#include "Interval.H"
#include "Stencils.H"
#include "BaseIVFAB.H"
#include "RedistStencil.H"

/// 
/**
  */
class EBLevelRedist
{
public:
  ///
  /**
     Default constructor.  User must subsequently call define().
  */
  EBLevelRedist();

  ///
  ~EBLevelRedist();

  ///
  /**
     Defining constructor.  Constructs a valid object.
     Equivalent to default construction followed by define().
     ebisl needs ghost cells out to 3*redistradius of the stencil.
     This sets the stencil to volume-weighted.
     Use resetWeights to set to mass weighted or whatever.
  */
  EBLevelRedist(const DisjointBoxLayout& a_dbl,
                const EBISLayout& a_ebisl,
                const Box& a_domain,
                const int& a_ncomp,
                int redistRad = 1);

  ///
  /**
     Defines this object.  Existing information is overriden.
     ebisl needs ghost cells out to 3*redistradius
  */
  void define(const DisjointBoxLayout& a_dbl,
              const EBISLayout& a_ebisl,
              const Box& a_domain,
              const int& a_ncomp,
              int redistRad = 1);

  ///
  /**
     Modify the weights in the stencil by multiplying by
     the inputs in a normalized way.
     If you want mass
     weighting, send in the density.
  */
  void resetWeights(const LevelData<EBCellFAB>& modifier,
                    const int& ivar);

  ///
  /**
     Store the input mass difference in the buffers of the class
     by INCREMENTING the buffer. \\
     dm += massDiff
   */
  void increment(const BaseIVFAB<Real>& massDiff,
                 const DataIndex& datInd,
                 const Interval& variables);

  ///
  /**
     Redistribute the data contained in the internal buffers. \\
     solution += weight*dm
  */
  void redistribute(LevelData<EBCellFAB>& solution,
                    const Interval& variables);
  ///
  /**
     Returns true if this object was created with the defining
     constructor or if define() has been called.
  */
  bool isDefined() const;

  ///
  /**
     Reset the mass buffers of the class.
   */
  void setToZero();

protected:
  int redistRad;
  RedistStencil m_stencil;
  int m_ncomp;
  int m_redistRad;
  DisjointBoxLayout  m_grids;
  Box m_domain;
  EBISLayout m_ebisl;
  bool m_isDefined;
  LevelData<BaseIVFAB<Real> > m_buffer;
  LayoutData<IntVectSet> m_sets;  
private:

  //forbidden for all the usual reasons
  void operator=(const EBLevelRedist& a_stenin)
  {
    MayDay::Error("Invalid operator");
  }

  EBLevelRedist(const EBLevelRedist& a_stenin)
  {
    MayDay::Error("Invalid operator");
  }
  
                           
};

#endif
