/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "EBQuadCFInterp.H"
#include "EBInterpolateF_F.H"
#include "VoFIterator.H"
#include "DisjointBoxLayout.H"
#include "EBCellFactory.H"
#include "LayoutIterator.H"
#include "EBIndexSpace.H"
#include "EBAlias.H"
/***********************/
bool 
EBQuadCFInterp::isDefined() const
{
  return m_isDefined;
}
/************************************/
EBQuadCFInterp::EBQuadCFInterp()
{
  m_isDefined = false;
  m_nComp = -1;
  m_refRat = -1;
}
/************************************/
EBQuadCFInterp::~EBQuadCFInterp()
{
}
/************************************/
EBQuadCFInterp::EBQuadCFInterp(const DisjointBoxLayout& a_dblFine,
                               const DisjointBoxLayout& a_coarBoxes,
                               const EBISLayout&        a_ebislFine,
                               const EBISLayout&        a_ebislCoar,
                               const Box&               a_domainCoar,
                               const int&               a_nref,
                               const int&               a_nvar)
{
  define(a_dblFine,   a_coarBoxes,  a_ebislFine, 
         a_ebislCoar, a_domainCoar, a_nref, a_nvar);
}
/************************************/
void 
EBQuadCFInterp::define(const DisjointBoxLayout& a_fineBoxes,
                       const DisjointBoxLayout& a_coarBoxes,
                       const EBISLayout&        a_ebislFine,
                       const EBISLayout&        a_ebislCoar,
                       const Box&               a_domainCoar,
                       const int&               a_nref,
                       const int&               a_nvar)
{
  m_isDefined = true;
  assert(a_nvar > 0);
  assert(a_nref > 0);
  assert(a_fineBoxes.isClosed());
  assert(a_coarBoxes.isClosed());
  assert(!a_domainCoar.isEmpty());

  m_nComp  = a_nvar;
  m_refRat = a_nref;
  m_domainCoar = a_domainCoar;
  m_domainFine = refine(m_domainCoar, m_refRat);
  m_gridsFine = a_fineBoxes;
  m_gridsCoar = a_coarBoxes;
  m_ebislFine = a_ebislFine;
  m_ebislCoar = a_ebislCoar;
  if(m_ebislFine.getMaxCoarseningRatio() < m_refRat)
    {
      m_ebislFine.setMaxCoarseningRatio(m_refRat);
    }
  Real dxFine = 1;
  ProblemDomain domFine(m_domainFine);
  m_quadCFInterp.define(m_gridsFine, &m_gridsCoar, dxFine, m_refRat, m_nComp, domFine);
}
/************************************/
void
EBQuadCFInterp::interpolate(LevelData<EBCellFAB>&       a_fineData,
                            const LevelData<EBCellFAB>& a_coarData,
                            const Interval&             a_variables)
{
  Interval totInterv(0, m_nComp-1);
  assert(a_fineData.nComp() == m_nComp);
  assert(a_coarData.nComp() == m_nComp);
  assert(a_variables.begin() >= 0);
  assert(a_variables.begin() < m_nComp);
  if(totInterv.size() != a_variables.size())
    {
      MayDay::Warning("EBQuadCFInterp variables mismatch");
    }
  LevelData<FArrayBox> fineDataLDFAB, coarDataLDFAB;

  aliasEB(fineDataLDFAB, a_fineData);
  aliasEB(coarDataLDFAB, (LevelData<EBCellFAB>&)a_coarData);
  m_quadCFInterp.coarseFineInterp(fineDataLDFAB, coarDataLDFAB);
}
/***********************/
