/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//
//
// an interface for I[NE]Fab data to Vince Beckner's ArrayView
//
#ifndef _EBARRAY_VIEW_CLIENT_H_
#define _EBARRAY_VIEW_CLIENT_H_

#include "REAL.H"
#include "LayoutData.H"
#include "LevelData.H"
#include "EBCellFAB.H"
#include "EBFaceFAB.H"
#include "BaseIFFAB.H"
#include "BaseIVFAB.H"
#include "EBArrayViewClient.H"


extern "C" 
{
  ///
  /**
     Arrayview interface for irregular data on faces.
     Can be called from debugger.
   */
  bool IFFABView(const BaseIFFAB<Real>* const fab);

  ///
  /**
     Arrayview interface for irregular data on VoFs.
     Can be called from debugger.
   */
  bool IVFABView(const BaseIVFAB<Real>* const fab);


  ///
  /**
     Arrayview interface for regular data on VoFs.
     Can be called from debugger.
   */
  bool EBCellFABView(const EBCellFAB* const fab);

  ///
  /**
     Arrayview interface for regular data on Faces.
     Can be called from debugger.
   */
  bool EBFaceFABView(const EBFaceFAB* const fab);
  bool IrregViewRealPtrArrayNVarDims(Real *data[], int nvar,    
                                     const int *lodim, const int *hidim,  
                                     const char *format, const char *label);

  ///
  /**
     Arrayview interface for regular data on VoFs.
     Can be called from debugger.
   */
  bool MultiEBCellFABView(const LayoutData<EBCellFAB >* const fab);

  ///
  /**
     Arrayview interface for regular data on VoFs.
     Do NOT call this one from the debugger.  Use
     only inside your program.
   */
  bool LDEBCellFABView(const LevelData<EBCellFAB >* const fab,
                       const Box* const a_domainPtr);
  void  makeFabFromEBFAB(BaseFab<Real>& fab,const EBCellFAB& ebfab);
}

#endif
