/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL, DTG

#ifndef _AllRegularSERVICE_H_
#define _AllRegularSERVICE_H_

#include "GeometryService.H"

/// A geometry service that is always regular.
/**
 */
class AllRegularService: public GeometryService
{
public:
  AllRegularService();

  virtual ~AllRegularService();

  /// 
  /** 
      Return true if every cell in region is regular at the
      refinement described by dx.
  */
  virtual bool isRegular(const Box& region,
                         const Box& domain,
                         const RealVect& origin,
                         const Real& dx) const;

  /// 
  /** 
      Return true if every cell in region is covered at the
      refinement described by dx.
  */
   virtual bool isCovered(const Box& region,
                          const Box& domain,
                          const RealVect& origin,
                          const Real& dx) const;

  ///
  /**
   */
  virtual void fillGraph(BaseFab<int>&        a_regIrregCovered,
                         Vector<IrregNode>&   a_nodes,
                         const Box&           a_validRegion,
                         const Box&           a_ghostRegion,
                         const Box&           a_domain,
                         const RealVect&      a_origin,
                         const Real&          a_dx) const;
};
#endif
