/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _BaseIVFactory_H_
#define _BaseIVFactory_H_

#include <cmath>
#include <cstdlib>
#include "Vector.H"
#include "EBISLayout.H"
#include "BaseIVFAB.H"
#include "DataIndex.H"

/// Factory class to produce BaseIVFABs.
/**
   Factory class to produce BaseIVFABs.
   This is needed for LevelData construction.
 */
template <class T>
class BaseIVFactory
  : public DataFactory<BaseIVFAB <T> >
{
public:
  /// factory function.
  /** 
      Creates a new baseivfab object 
      and returns a pointer to it.  Responsiblitly
      for calling operator 'delete' on this pointer 
      is passed to the user. 
      Intersects a_sets[a_dit] from construction
      with input box because leveldata will expect
      that anyway when it tries to do a copy or a linearization.
  */
  virtual BaseIVFAB<T>* create(const Box& a_box, int a_ncomps,
                               const DataIndex& a_dit) const;

  ///
  /**
     fills internal data with inputs
  */
  BaseIVFactory(const EBISLayout& a_ebisl, 
                const LayoutData<IntVectSet>& a_sets);

  ///
  virtual ~BaseIVFactory();

private:
  ///
  EBISLayout m_ebisl;
  
  ///
  LayoutData<IntVectSet> m_sets;

  //disallowed constructors and all that
  BaseIVFactory(const BaseIVFactory<T>& a_inputs)
  {
    MayDay::Error("disallowed constructor");
  }
  BaseIVFactory()
  {
    MayDay::Error("disallowed constructor");
  }
  void operator=(const BaseIVFactory<T>& a_inputs)
  {
    MayDay::Error("disallowed constructor");
  }

};
//implementation
#include "BaseIVFactoryI.H"

#endif
