/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _CoveredFaceFAB_H_
#define _CoveredFaceFAB_H_

#include "IntVect.H"
#include "VolIndex.H"
#include "IntVectSet.H"
#include "EBISBox.H"
#include "BaseIVFAB.H"
#include "Vector.H"
#include "LoHiSide.H"


///holder for data over covered faces
class CoveredFaceFAB
{
public:

  ///
  int nComp() const;

  ///
  int getIndex(int a_index, Side::LoHiSide a_sd) const;

  ///
  void clear();

  ///
  CoveredFaceFAB();

  ///
  ~CoveredFaceFAB();

  ///
  CoveredFaceFAB(const IntVectSet&  a_ivs,
                 const EBISBox&     a_ebisBox,
                 int                a_nvar);

  ///
  void define(const IntVectSet&  a_ivs,
              const EBISBox&     a_ebisBox,
              int                a_nvar);

  ///
  BaseIVFAB<Real>&  operator() 
    (int a_idir, Side::LoHiSide a_sd);

  ///
  const BaseIVFAB<Real>& operator()
    (int a_idir, Side::LoHiSide a_sd) const;

  ///
  const IntVectSet& getIVS(int a_idir, Side::LoHiSide a_sd) const;

  ///
  bool isDefined() const;

  ///
  void setVal(const Real& a_val);

  ///
  void 
  copy(const Box& Rfrom, const Interval& Cdest, 
       const Box& Rto, const CoveredFaceFAB& src,
       const Interval& Csrc);

private:
  
  CoveredFaceFAB(const CoveredFaceFAB& a_input)
  {
    MayDay::Error("Invalid Operator");
  }
  void
  operator= (const CoveredFaceFAB& a_input)
  {
    MayDay::Error("Invalid Operator");
  }

  EBISBox          m_ebisBox;
  IntVectSet       m_sets[2*SpaceDim];
  BaseIVFAB<Real>  m_data[2*SpaceDim];
  int              m_nComp;
  bool             m_isDefined;
  
};


#endif
