/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _CoveredFaceIterator_H_
#define _CoveredFaceIterator_H_

#include "IntVect.H"
#include "VolIndex.H"
#include "IntVectSet.H"
#include "EBISBox.H"
#include "Vector.H"
#include "LoHiSide.H"

/// Iterator for all vofs within an IntVectSet and an EBISBox
/**
   Iterator for all vofs within an IntVectSet and an EBISBox
 */
class CoveredFaceIterator
{
public:
  ///
  CoveredFaceIterator(const IntVectSet&         a_ivs,
                      const EBISBox&            a_ebisBox,
                      int a_dir, Side::LoHiSide a_side);

  ///
  CoveredFaceIterator();

  ///
  ~CoveredFaceIterator();

  ///
  void define(const IntVectSet& a_ivs,
              const EBISBox& a_ebisBox,
              int a_dir, Side::LoHiSide a_side);

  ///
  void reset();

  ///
  void operator++();

  ///
  const VolIndex& operator() () const;

  ///
  bool ok() const;

  ///
  bool isDefined() const;

private:
  CoveredFaceIterator(const CoveredFaceIterator& a_input)
  {
    MayDay::Error("Invalid Operator");
  }
  void
  operator= (const CoveredFaceIterator& a_input)
  {
    MayDay::Error("Invalid Operator");
  }
  Vector<VolIndex> m_vols;
  int m_ivol;

  bool m_isDefined;
};

#endif

