/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "EBDebugOut.H"
#include "LayoutIterator.H"
#include "parstream.H"
#include "FaceIndex.H"
#include "Vector.H"
#include "VolIndex.H"
#include "LoHiSide.H"
#include "VoFIterator.H"
#include <iostream>
using std::cout;
using std::cin;
using std::cerr;
using std::endl;
#include <iomanip>

void dumpIVFAB(const BaseIVFAB<Real>* a_vectPtr)
{
  const BaseIVFAB<Real>& ivfab = *a_vectPtr;
  const int ncomp = ivfab.nComp();
  const EBGraph& ebgraph = ivfab.getEBGraph();
  const IntVectSet& ivs = ivfab.getIVS();

  pout() << "data in base ivfab" << endl;
  for(VoFIterator vofit(ivs, ebgraph); vofit.ok(); ++vofit)
    {
      const VolIndex& vof = vofit();
      pout() << "vof= " << vof.gridIndex();
      //RealVect centroid = ebgraph.centroid(vof);
      //pout() << ";      cent=";
      //for(int idir = 0; idir < SpaceDim; idir++)
      //  {
      //    pout() << " " 
      //           << setprecision(4)
      //           << setiosflags(ios::showpoint)
      //           << setiosflags(ios::scientific)
      //           << centroid[idir];
      //  }
      pout() << ";      data=";
      for(int ivar = 0; ivar < ncomp; ivar++)
        {
          pout() << " " 
                 << setprecision(4)
                 << setiosflags(ios::showpoint)
                 << setiosflags(ios::scientific)
                 << ivfab(vof, ivar);
        }
      pout() << endl;
    }
}
void dumpIFFAB(const BaseIFFAB<Real>* a_vectPtr)
{
  const BaseIFFAB<Real>& iffab = *a_vectPtr;
  const int ncomp = iffab.nComp();
  const int dir = iffab.direction();
  const EBGraph& ebgraph = iffab.getEBGraph();
  const IntVectSet& ivs = iffab.getIVS();
  FaceIterator faceit(ivs, ebgraph, dir, 
                      FaceStop::SurroundingWithBoundary); 
  pout() << "data in base iffab" << endl;
  for(faceit.reset();faceit.ok(); ++faceit)
    {
      const FaceIndex& face = faceit();
      pout() << "face= ";
      for(SideIterator sit; sit.ok(); ++sit)
        {
          pout() << " " << face.gridIndex(sit());
        }
      for(int ivar = 0; ivar < ncomp; ivar++)
        {
          pout() << " " 
                 << setprecision(4)
                 << setiosflags(ios::showpoint)
                 << setiosflags(ios::scientific)
                 << iffab(face, ivar);
        }
      pout() << endl;
    }
}
void dumpVVoF(const Vector<VolIndex>* a_vectPtr)
{
  if(a_vectPtr == NULL) return;
  Vector<VolIndex> vect = *a_vectPtr;
  pout() << "vector of vofs contains" << endl;
  for(int iveco = 0; iveco < vect.size(); iveco++)
    {
      const VolIndex& vof = vect[iveco];
      pout() <<  vof.gridIndex() << "   "  << vof.cellIndex() << "  ";;
    }
  pout() << endl;
}

void dumpVFace(const Vector<FaceIndex>* a_vectPtr)
{
  if(a_vectPtr == NULL) return;
  Vector<FaceIndex> vect = *a_vectPtr;
  pout() << "vector of faces contains" << endl;
  for(int iveco = 0; iveco < vect.size(); iveco++)
    {
      for(SideIterator sit; sit.ok(); ++sit)
        {
          VolIndex vof = vect[iveco].getVoF(sit());
          pout() <<  vof.gridIndex() << "   " << vof.cellIndex() << "  ";;
        }
      if(vect[iveco].isBoundary())
        {
          pout() <<  "on boundary";
        }
      pout() << endl;
    }
}

void dumpFace(const FaceIndex* a_vectPtr)
{
  if(a_vectPtr == NULL) return;
  const FaceIndex& face = *a_vectPtr;
  pout() << "faces contains" << endl;
  for(SideIterator sit; sit.ok(); ++sit)
    {
      VolIndex vof = face.getVoF(sit());
      pout() <<  vof.gridIndex() << "   " << vof.cellIndex() << "  ";
    }
  pout() << endl;
  if(face.isBoundary())
    {
      pout() <<  " face is on on boundary";
      pout() << endl;
    }
}

void dumpFaceSten(const FaceStencil* a_vectPtr)
{
  if(a_vectPtr == NULL) return;
  const FaceStencil& sten = *a_vectPtr;
  pout() << "stencil contains:" << endl;
  for(int isten = 0; isten <  sten.size(); isten++)
    {
      const FaceIndex& face = sten.face(isten);
      const Real& weight = sten.weight(isten);
      pout() << "face: ";
      for(SideIterator sit; sit.ok(); ++sit)
        {
          VolIndex vof = face.getVoF(sit());
          pout() <<  vof.gridIndex() << "   " << vof.cellIndex() << "  ";;
        }
      pout() << "weight: " << weight;
      pout() << endl;
    }
}

void dumpVoFSten(const VoFStencil* a_vectPtr)
{
  if(a_vectPtr == NULL) return;
  const VoFStencil& sten = *a_vectPtr;
  pout() << "stencil contains:" << endl;
  for(int isten = 0; isten <  sten.size(); isten++)
    {
      const VolIndex& vof = sten.vof(isten);
      const Real& weight = sten.weight(isten);
      pout() << "vof: ";
      pout() <<  vof.gridIndex() << "   " << vof.cellIndex() << "  ";;
      pout() << "weight: " << weight;
      pout() << endl;
    }
}
