/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _EBFaceFAB_H_
#define _EBFaceFAB_H_

#include <cmath>
#include <cstdlib>
#include <cassert>
#include "BaseEBFaceFAB.H"
#include "FArrayBox.H"

/// 
/**
   EBFaceFAB is a BaseEBFaceFAB<Real> 
   which includes arithmetic functions.
 */
class EBFaceFAB: public BaseEBFaceFAB<Real>
{
public:
  ///
  EBFaceFAB();


  ///
  /**
    Box going into this needs to be cell-centered.
    Data will exist over the surrounding nodes of the box.
   */
  EBFaceFAB(const EBISBox& a_ebisBox,
            const Box& a_region, 
            int a_iDir, int a_nVar);


  ///
  virtual ~EBFaceFAB();


  ///
  const FArrayBox& getFArrayBox() const;

  ///
  /**
     Does a dynamic cast of the BaseFab<Real>
     to a  FArrayBox for arithmetic purposes.
   */
  FArrayBox& getFArrayBox();

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator+=(const EBFaceFAB& a_ebfab);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator-=(const EBFaceFAB& a_ebfab);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator*=(const EBFaceFAB& a_ebfab);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator/=(const EBFaceFAB& a_ebfab);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator+=(const Real& a_scalar);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator-=(const Real& a_scalar);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator*=(const Real& a_scalar);

  ///
  /**
     Both fabs need the same ebisBox and region.
   */
  EBFaceFAB& operator/=(const Real& a_scalar);

  //needed for leveldata to compile but invalid
  EBFaceFAB(const Box& a_region, int a_nComp)
  {
    MayDay::Error("bogus constructor");
  }
private:
  //verboten for all the usual reasons
  EBFaceFAB(const EBFaceFAB& ebcin)
  {
    MayDay::Error("bogus constructor");
  }
  void operator=(const EBFaceFAB& ebcin)
  {
    MayDay::Error("bogus constructor");
  }

};
#endif

