/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// EBFluxFAB.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// DTG 7-31-2001


#include "EBFluxFAB.H"

// first do simple access functions
// ---------------------------------------------------------
int 
EBFluxFAB::nComp() const 
{ 
  return m_nComp;
}

// ---------------------------------------------------------
const Box& 
EBFluxFAB::getRegion() const 
{
  return m_region;
}


// ---------------------------------------------------------
EBFaceFAB&
EBFluxFAB::operator[] (const int dir) 
{
  assert(m_nComp >0);
  assert(dir < SpaceDim);
  assert(m_fluxes[dir] != NULL);

  return *m_fluxes[dir];
}  


// ---------------------------------------------------------
const EBFaceFAB&
EBFluxFAB::operator[] (const int dir)  const 
{
  assert(m_nComp >0);
  assert(dir < SpaceDim);
  assert(m_fluxes[dir] != NULL);

  return *m_fluxes[dir];
}  


// ---------------------------------------------------------
// constructors and destructors
// ---------------------------------------------------------
EBFluxFAB::EBFluxFAB()
{
  setDefaultValues();
}

// ---------------------------------------------------------
EBFluxFAB::EBFluxFAB(const EBISBox& a_ebisBox,
                     const Box& a_region, int a_nComp)  
{
  setDefaultValues();
  define(a_ebisBox, a_region, a_nComp);
}
void
EBFluxFAB::define(const EBISBox& a_ebisBox,
                  const Box& a_region, int a_nComp)  
{
  clear();
  m_isDefined = true;
  m_ebisBox = a_ebisBox;
  m_region = a_region & a_ebisBox.getRegion();
  m_nComp = a_nComp;
  for(int idir = 0; idir < SpaceDim; idir++)
    {
      m_fluxes[idir] = new EBFaceFAB(a_ebisBox, a_region, idir, a_nComp);
    }
}
// ---------------------------------------------------------
bool 
EBFluxFAB::isDefined() const
{
  return m_isDefined;
}
// ---------------------------------------------------------
EBFluxFAB::~EBFluxFAB()
{

  clear();
}

// ---------------------------------------------------------
void 
EBFluxFAB::clear() 
{
  // first delete storage
  for (int dir=0; dir < SpaceDim; dir++) 
    {
      if (m_fluxes[dir] != NULL) 
        {
          delete m_fluxes[dir];
        }
    }

  // now reset all other variables
  setDefaultValues();
}
    
void
EBFluxFAB::setDefaultValues()
{
  for (int dir=0; dir < SpaceDim; dir++) 
    {
      m_fluxes[dir] = NULL;
    }

  // set the box to the empty box...
  m_region = Box();
}

// ---------------------------------------------------------
void
EBFluxFAB::setVal(const Real& val)
{
  assert(m_nComp > 0);

  for (int dir = 0; dir < SpaceDim; dir++) 
    {
      assert(m_fluxes[dir] != NULL);
      m_fluxes[dir]->setVal(val);
    }
}


// ---------------------------------------------------------
void 
EBFluxFAB::copy(const Box& Rfrom, const Interval& Cdest, 
                const Box& Rto, const EBFluxFAB& src,
                const Interval& Csrc)
{
  assert(Rfrom == Rto);
  Box R = Rto;
  for (int dir=0; dir<SpaceDim; dir++) 
    {
      const EBFaceFAB& srcFab = src[dir];
      m_fluxes[dir]->copy(R,Cdest,R, srcFab,Csrc);
    }
  
}



