/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _FaceIterator_H_
#define _FaceIterator_H_

#include "IntVect.H"
#include "FaceIndex.H"
#include "IntVectSet.H"
#include "EBGraph.H"
#include "Vector.H"

///
/**
   Ennumeration class to distinguish where FaceIterator will stop.
 */
class FaceStop
{
public:
  ///
  /**
     SurroundingWithBoundary  means stop at all faces on the high and
     low sides of IVS cells. \\
     SurroundingNoBoundary  means stop at all faces on the high and
     low sides of IVS cells, excluding faces on the domain boundary.\\
     LoWithBoundary  means stop at all faces on the 
     low side of IVS cells.\\
     LoNoBoundary  means stop at all faces on the
     low side of IVS cells, excluding faces on the domain boundary.\\
     HiWithBoundary  means stop at all faces on the 
     high side of IVS cells.\\
     LoNoBoundary  means stop at all faces on the
     high side of IVS cells, excluding faces on the domain boundary.\\
  */
  enum WhichFaces{Invalid=-1, 
                  SurroundingWithBoundary=0, HiWithBoundary, LoWithBoundary, 
                  SurroundingNoBoundary    , HiNoBoundary  , LoNoBoundary, 
                  LoBoundaryOnly, HiBoundaryOnly, AllBoundaryOnly,
                  NUMTYPES};
};

/// Iterator over faces within an IntVectSet and an Ebgraph
/**
   Iterator over within an IntVectSet and an Ebgraph.
   The enumeration tells which faces on which to stop.
 */
class FaceIterator
{
public:

  ///
  FaceIterator(const IntVectSet& a_ivs,
               const EBGraph& a_ebgraph,
               const int& a_direction,
               const FaceStop::WhichFaces& a_location);

  ///
  FaceIterator();

  ///
  ~FaceIterator();

  ///
  void define(const IntVectSet& a_ivs,
              const EBGraph& a_ebgraph,
              const int& a_direction,
              const FaceStop::WhichFaces& a_location);

  ///
  void reset();

  ///
  void operator++();

  ///
  const FaceIndex& operator() () const;

  ///
  bool ok() const;

  ///
  bool isDefined() const;

  ///
  const Vector<FaceIndex>& getVector() const;

private:
  bool m_isDefined;
  Vector<FaceIndex> m_faces;
  int m_iface;
  int m_direction;
};

#endif
