/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#include "FaceIterator.H"
#include "BaseIFFAB.H"
/********************************/
const Vector<FaceIndex>& 
FaceIterator::getVector() const
{
  return m_faces;
}
/********************************/
FaceIterator::FaceIterator()
{
  m_isDefined = false;
}
/********************************/
FaceIterator::~FaceIterator()
{
}
/********************************/
FaceIterator::FaceIterator(const IntVectSet& a_ivs,
                           const EBGraph& a_ebgraph,
                           const int& a_direction,
                           const FaceStop::WhichFaces& a_location)
{
  define(a_ivs, a_ebgraph, a_direction, a_location);
}
/********************************/
void
FaceIterator::define(const IntVectSet& a_ivs,
                     const EBGraph& a_ebgraph,
                     const int& a_direction,
                     const FaceStop::WhichFaces& a_location)
{
  //can't do this because minbox is broken
  //  assert((a_ebgraph.getRegion().contains(a_ivs.minBox())||
  //          a_ivs.isEmpty()));
  m_isDefined = true;
  m_direction = a_direction;
  m_faces.resize(0);
  //this kruft is to keep a face from being added twice
  BaseIFFAB<bool> doneThat(a_ivs, a_ebgraph, a_direction, 1);
  doneThat.setVal(false);
  bool doLo = ((a_location == FaceStop::SurroundingNoBoundary) ||
               (a_location == FaceStop::SurroundingWithBoundary) ||
               (a_location == FaceStop::LoWithBoundary) ||
               (a_location == FaceStop::LoBoundaryOnly) ||
               (a_location == FaceStop::LoNoBoundary));
  bool doHi = ((a_location == FaceStop::SurroundingNoBoundary) ||
               (a_location == FaceStop::SurroundingWithBoundary) ||
               (a_location == FaceStop::HiWithBoundary) ||
               (a_location == FaceStop::HiBoundaryOnly) ||
               (a_location == FaceStop::HiNoBoundary));
  bool doBoundary = ((a_location == FaceStop::HiWithBoundary) ||
                     (a_location == FaceStop::SurroundingWithBoundary) ||
                     (a_location == FaceStop::LoWithBoundary) ||
                     (a_location == FaceStop::LoBoundaryOnly) ||
                     (a_location == FaceStop::HiBoundaryOnly) ||
                     (a_location == FaceStop::AllBoundaryOnly));

  bool boundaryOnly = (a_location == FaceStop::AllBoundaryOnly);
  //if this fails, invalid location.
  assert(doLo || doHi || boundaryOnly);

  for(IVSIterator ivsit(a_ivs); ivsit.ok(); ++ivsit)
    {
      
      Vector<FaceIndex> faces;
      if(doLo || boundaryOnly)
        {
          faces.append(a_ebgraph.getAllFaces(ivsit(), 
                                             a_direction, 
                                             Side::Lo));
        }
      if(doHi || boundaryOnly)
        {
          faces.append(a_ebgraph.getAllFaces(ivsit(), 
                                             a_direction, 
                                             Side::Hi));
        }
      for(int iface = 0; iface < faces.size(); iface++)
        {
          const FaceIndex& face = faces[iface];
          if(!doneThat(face, 0))
            {
              doneThat(face,0) = true;
              //there has to be a better way to do this logic
              if(boundaryOnly && face.isBoundary())
                {
                  m_faces.push_back(face);
                }
              else if(!face.isBoundary() || doBoundary)
                {
                  m_faces.push_back(face);
                }
            }
        }//end loop over faces in cell at a side
    }//end loop over cells in IVS
  reset();
}

/********************************/
void 
FaceIterator::reset()
{
  assert(isDefined());
  m_iface = 0;
}


/********************************/
//keys off if icell >= m_cellVols.size()
/********************************/
void 
FaceIterator::operator++()
{
  assert(isDefined());
  m_iface++;
}


/********************************/
const FaceIndex& 
FaceIterator::operator() () const
{
  assert(isDefined());
  assert(m_iface < m_faces.size());
  return m_faces[m_iface];
}


/********************************/
bool 
FaceIterator::ok() const
{
  return (m_iface < m_faces.size());
}

/********************************/
bool 
FaceIterator::isDefined() const
{
  return m_isDefined;
}

