/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _IrregNodeNode_H_
#define _IrregNodeNode_H_

#include "REAL.H"
#include "IntVect.H"
#include "IntVectSet.H"
#include "VolIndex.H"
#include "Interval.H"
#include "FaceIndex.H"
#include "IntVectSet.H"
#include "RealVect.H"
#include "BaseFab.H"
#include "SPMD.H"
///
/**
   Node for construction of geometric information.
*/
class IrregNode
{
public:
  ///
  IrregNode();

  ///
  ~IrregNode();

  ///return the index into the arc vectors
  int index(int a_idir, Side::LoHiSide a_side) const;

  ///
  IntVect     m_cell;
  ///data for irregular nodes

  //volume centered data
  ///
  Real         m_volFrac;

  ///
  /**
     each vof has a unique index in the cell
  */
  int          m_cellIndex;

  ///
  RealVect     m_volCentroid;

  ///
  RealVect     m_bndryCentroid;

  //face centered irregular data

  ///
  /**
     Indicies into a_nodes to show connectivity.  
     If the arc is to  an irregular
     cell, the index is the unique index of the vof
     in the cell. 
     For arcs to regular cells, the arc = -2
     If the arc is to the domain boundary, arc  = -1.
  */
  Vector<int>      m_arc[2*SpaceDim];

  ///
  Vector<Real>     m_areaFrac[2*SpaceDim];

  ///
  Vector<RealVect> m_faceCentroid[2*SpaceDim];

  
  ///
  /** 
	  helper function for construction.  makes irregular cell
	  that has connectivitity and values like a regular cell, this
	  a person can modify as the irregular cell requires.  saves
	  some coding in some cases
  */
  void makeRegular(const IntVect& iv); 
  void faceReserve(int location, int size);
};
#endif
