/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#include "IrregNode.H"

/*******************************/
/*******************************/
IrregNode::IrregNode()
{
}
/*******************************/
/*******************************/
IrregNode::~IrregNode()
{
}
/*******************************/
/*******************************/
int IrregNode::
index(int a_idir, Side::LoHiSide a_sd) const
{
  assert(a_idir >= 0 && a_idir < SpaceDim);
  int retval;
  if(a_sd == Side::Lo)
    {
      retval = a_idir;
    }
  else
    {
      retval = a_idir + SpaceDim;
    }
  return retval;
}
/*******************************/
/*******************************/

void IrregNode::makeRegular(const IntVect& iv)
{
  m_cell = iv;
  m_volFrac = 1.0;
  m_cellIndex = 0;
  m_volCentroid = IntVect::Zero;
  m_bndryCentroid = IntVect::Zero;
  //low sides
  for(int i=0; i<SpaceDim; i++)
    {
      m_arc[i].resize(1,0);
      m_areaFrac[i].resize(1,1.0);
      RealVect faceCenter = IntVect::Zero;
      faceCenter[i] = -0.5;
      m_faceCentroid[i].resize(1,faceCenter);
    }
  //hi sides
  for(int i=0; i<SpaceDim; i++)
    {
      m_arc[i+SpaceDim].resize(1,0);
      m_areaFrac[i+SpaceDim].resize(1,1.0);
      RealVect faceCenter = IntVect::Zero;
      faceCenter[i] = 0.5;
      m_faceCentroid[i+SpaceDim].resize(1,faceCenter);
    }
}

void IrregNode::faceReserve(int location, int size)
{
  if(m_arc[location].size() < size)
    {
      m_arc[location].resize(size);
      m_areaFrac[location].resize(size);
      m_faceCentroid[location].resize(size);
    }
}
