/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL, DTG

#ifndef _SLABSERVICE_H_
#define _SLABSERVICE_H_

#include <cmath>
#include <cstdlib>
#include "Vector.H"
#include "Box.H"
#include "EBISBox.H"
#include "GeometryService.H"
/// 
/**
   This is a pure base class that EBIndexSpace uses
   to interface with geometry-generation classes.
 */
class SlabService: public GeometryService
{
public:
  ///
  SlabService(const Box& a_coveredRegion);

  virtual ~SlabService();

  /// 
  /** 
      Return true if every cell in region is regular at the
      refinement described by dx.
  */
  virtual bool isRegular(const Box&      a_region,
                         const Box&      a_domain,
                         const RealVect& a_origin,
                         const Real&     a_dx) const;

  /// 
  /** 
      Return true if every cell in region is covered at the
      refinement described by dx.
  */
  virtual bool isCovered(const Box&      a_region,
                         const Box&      a_domain,
                         const RealVect& a_origin,
                         const Real&     a_dx) const;

  ///
  /**
     irregGraph contains a complete list of irregular vofs. \\
     validRegion is the the region of the graph \\
     domain is the domain of compuation \\
     regIrregCovered = 1 for regular, 0 for irregular, -1 for covered.
     regIrregCovered must contain the valid region grown by 1. \\
     In chombospeak, \\
     ghostregion = (grow(validRegion, 1) & domain); 
   */
  virtual void fillGraph(BaseFab<int>&        a_regIrregCovered,
                         Vector<IrregNode>&   a_nodes,
                         const Box&           a_validRegion,
                         const Box&           a_ghostRegion,
                         const Box&           a_domain,
                         const RealVect&      a_origin,
                         const Real&          a_dx) const;


protected:
  Box m_coveredRegion;

};
#endif
