/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "Stencils.H"
/**************/
/**************/
VoFStencil::VoFStencil()
  : vofs(0),
    weights(0)
{
}
/**************/
/**************/
void
VoFStencil::clear()
{
  vofs.resize(0);
  weights.resize(0);
}
/**************/
/**************/
void
FaceStencil::clear()
{
  faces.resize(0);
  weights.resize(0);
}

/**************/
/**************/
VoFStencil::VoFStencil(const VoFStencil&  stenin)
  : vofs(stenin.vofs),
    weights(stenin.weights)
{
}
/**************/
/**************/
VoFStencil::~VoFStencil() 
{
}
/**************/
/**************/
void
VoFStencil::add(const VolIndex& vof,const Real& weight)
{
  bool alreadyhere = false;
  for(int ivof = 0; ivof < vofs.size(); ivof++)
    {
      if(vofs[ivof] == vof)
        {
          alreadyhere = true;
          weights[ivof] += weight;
        }
    }
  if(!alreadyhere)
    {
      vofs.push_back(vof);
      weights.push_back(weight);
    }
}
/**************/
/**************/
VoFStencil&
VoFStencil::operator+=(const VoFStencil& vofstenin)
{
  for(int ivof = 0; ivof < vofstenin.size(); ivof++)
    {
      add(vofstenin.vof(ivof), vofstenin.weight(ivof));
    }
  return *this;
}
/**************/
/**************/
void 
VoFStencil::operator*=(const Real& a_scaling)
{
  for(int ivof = 0; ivof < size(); ivof++)
    {
      weights[ivof] *= a_scaling;
    }
}
/**************/
/**************/
void 
FaceStencil::operator*=(const Real& a_scaling)
{
  for(int iface = 0; iface < size(); iface++)
    {
      weights[iface] *= a_scaling;
    }
}
/**************/
/**************/
VoFStencil&
VoFStencil::operator=(const VoFStencil& vofstenin)
{
  clear();
  this->operator+=(vofstenin);
  return *this;
}
/**************/
/**************/
int
VoFStencil::size() const
{
  return weights.size();
}
/**************/
/**************/
const  VolIndex&
VoFStencil::vof(int i) const
{
  return vofs[i];
}
/**************/
/**************/
const Real&
VoFStencil::weight(int i) const
{
  return weights[i];
}
/**************/
/**************/
FaceStencil::FaceStencil()
  : faces(0),
    weights(0)
{
}
/**************/
/**************/
FaceStencil::FaceStencil(const FaceStencil& facestenin)
  : faces(facestenin.faces),
  weights(facestenin.weights)
{
}
/**************/
/**************/
// destructor
FaceStencil::~FaceStencil() 
{
}
/**************/
/**************/
void
FaceStencil::add(const FaceIndex& face,const Real& weight)
{
  bool alreadyhere = false;
  for(int iface = 0; iface < faces.size(); iface++)
    {
      if(faces[iface] == face)
        {
          alreadyhere = true;
          weights[iface] += weight;
        }
    }
  if(!alreadyhere)
    {
      faces.push_back(face);
      weights.push_back(weight);
    }
}
/**************/
/**************/
int
FaceStencil::size() const
{
  return weights.size();
}
/**************/
/**************/
const FaceIndex&
FaceStencil::face(int i) const
{
  return faces[i];
}
/**************/
/**************/
const Real&
FaceStencil::weight(int i) const
{
  return weights[i];
}

/**************/
/**************/
FaceStencil&
FaceStencil::operator+=(const FaceStencil& Facestenin)
{
  for(int iFace = 0; iFace < Facestenin.size(); iFace++)
    {
      add(Facestenin.face(iFace), Facestenin.weight(iFace));
    }
  return *this;
}
/**************/
/**************/
FaceStencil&
FaceStencil::operator=(const FaceStencil& Facestenin)
{
  clear();
  this->operator+=(Facestenin);
  return *this;
}
