/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#include "VoFIterator.H"

/********************************/
const Vector<VolIndex> 
VoFIterator::getVector() const
{
  assert(m_isDefined);
  return m_vols;
}
/********************************/
VoFIterator::VoFIterator()
{
  m_isDefined = false;
}
/********************************/
VoFIterator::~VoFIterator()
{
}
/********************************/
VoFIterator::VoFIterator(const IntVectSet& a_ivs,
                         const EBGraph& a_ebgraph)
{
  define(a_ivs, a_ebgraph);
}
/********************************/
void
VoFIterator::define(const IntVectSet& a_ivs,
                    const EBGraph& a_ebgraph)
{
  //can't do this because minbox is broken
  //  assert((a_ebgraph.getRegion().contains(a_ivs.minBox())||
  //          a_ivs.isEmpty()));
  m_isDefined = true;
  m_vols.resize(0);
  for(IVSIterator ivsit(a_ivs); ivsit.ok(); ++ivsit)
    {
      m_vols.append(a_ebgraph.getVoFs(ivsit()));
    }
  reset();
}
/********************************/
void 
VoFIterator::reset()
{
  assert(isDefined());
  m_ivol = 0;
}
/********************************/
void 
VoFIterator::operator++()
{
  assert(isDefined());
  m_ivol++;
}
/********************************/
const VolIndex& 
VoFIterator::operator() () const
{
  assert(isDefined());
  assert(m_ivol < m_vols.size());
  return m_vols[m_ivol];
}
/********************************/
bool 
VoFIterator::ok() const
{
  return (m_ivol < m_vols.size());
}
/********************************/
bool 
VoFIterator::isDefined() const
{
  return m_isDefined;
}
/********************************/

