/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#include "VolIndex.H"
#include "parstream.H"
#include "CH_HDF5.H"

using std::ostream;


size_t VolIndex::iv_offset =  HOFFSET(VolIndex, m_iv.vect);
size_t VolIndex::index_offset = HOFFSET(VolIndex, m_cellIndex);

using std::endl;

int VolIndex::linearSize() const
{
  return (CH_SPACEDIM + 1)*sizeof(int);
}

void VolIndex::linearOut(void* const a_outBuf) const
{
  assert(m_isDefined);
  int* buf = (int*)a_outBuf;
  D_TERM(buf[0]=m_iv[0],; buf[1]=m_iv[1],; buf[2]=m_iv[2]);
  buf[CH_SPACEDIM]=m_cellIndex;

}

void VolIndex::linearIn(const void* const inBuf)
{
  int* buf = (int*)inBuf;
  D_TERM(m_iv[0]=buf[0],; m_iv[1]=buf[1],; m_iv[2]=buf[2]);
  m_cellIndex = buf[CH_SPACEDIM];
  m_isDefined = true;
}

/*****************************************/
ostream&
operator<< (ostream&       os,
            const VolIndex& p)
{
  IntVect iv = p.gridIndex();
  int vofind = p.cellIndex();
  os << D_TERM( "((" << iv[0] , <<
                ',' << iv[1] , <<
                ',' << iv[2])  << ")(" << vofind << "))" ;
  if (os.fail())
    MayDay::Error("operator<<(ostream&,VolIndex&) failed");
  return os;
}
/*****************************************/
int VolIndex::initializeOffsets()
{
  iv_offset = offsetof(VolIndex, m_iv.vect);
  index_offset = offsetof(VolIndex, m_cellIndex);
  return 1;
}

static int init =  VolIndex::initializeOffsets();


/*****************************************/
void
VolIndex::define(const VolIndex& a_vofin)
{
  //must be allowed to propogate undefinednitude
  //because of vector<vol>
  //  assert(a_vofin.isDefined());

  m_isDefined = a_vofin.m_isDefined;
  m_iv = a_vofin.m_iv;
  m_cellIndex = a_vofin.m_cellIndex;
}



/****************************************
VolIndex::~VolIndex()
{;}
*/




/*****************************************/

