/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// BVS, June 18, 2003

// We can assume that template class T has null construction.

#ifndef _AMRPOISSONOP_H_
#define _AMRPOISSONOP_H_

#include "AMRMultiGrid.H"
#include "REAL.H"
#include "Box.H"
#include "LevelDataOps.H"
#include "BCFunc.H"
#include "FArrayBox.H"
#include "NewCFIVS.H"
#include "QuadCFInterp.H"



class AMRPoissonOp : public AMRLevelOp<LevelData<FArrayBox> >
{
public:
  AMRPoissonOp(){ ; }

  virtual ~AMRPoissonOp() { ; }
  /**
     \name LinearOp functions */
  /*@{*/

  void define(const DisjointBoxLayout& a_grids, const RealVect& a_dx,
              const ProblemDomain& a_domain, BCFunc a_bc);

  void define(const DisjointBoxLayout& a_grids,
              const DisjointBoxLayout& a_baseBAPtr,
              const RealVect&          a_dxLevel,
              int                      a_refRatio,
              const ProblemDomain&     a_domain,
              BCFunc                   a_bc,
              int                      a_ncomp);

  virtual void residual(  LevelData<FArrayBox>& a_lhs,
                          const LevelData<FArrayBox>& a_phi,
                          const LevelData<FArrayBox>& a_rhs,
                          bool a_homogeneous = false);

  virtual void preCond(   LevelData<FArrayBox>& a_correction,
                          const LevelData<FArrayBox>& a_residual);

  virtual void applyOp(   LevelData<FArrayBox>& a_lhs,
                          const LevelData<FArrayBox>& a_phi,
                          bool a_homogeneous = false);
  virtual void create(    LevelData<FArrayBox>& a_lhs,
                          const LevelData<FArrayBox>& a_rhs);
  virtual void assign(    LevelData<FArrayBox>& a_lhs,
                          const LevelData<FArrayBox>& a_rhs) ;
  virtual Real dotProduct(const LevelData<FArrayBox>& a_1,
                          const LevelData<FArrayBox>& a_2) ;
  virtual void incr( LevelData<FArrayBox>& a_lhs,
                     const LevelData<FArrayBox>& a_x,
                     Real a_scale) ;
  virtual void axby( LevelData<FArrayBox>& a_lhs, const LevelData<FArrayBox>& a_x,
                     const LevelData<FArrayBox>& a_y,
                     Real a, Real b) ;
  virtual void scale(LevelData<FArrayBox>& a_lhs, const Real& a_scale) ;

  virtual Real norm(const LevelData<FArrayBox>& a_x, int a_ord);

  virtual void setToZero( LevelData<FArrayBox>& a_x);
  /*@}*/

  /**
     \name MGLevelOp functions */
  /*@{*/

  virtual void relax(LevelData<FArrayBox>& a_e,
                     const LevelData<FArrayBox>& a_residual,
                     int iterations);

  virtual void createCoarser(LevelData<FArrayBox>& a_coarse,
                             const LevelData<FArrayBox>& a_fine,
                             bool ghosted);
  /**
     calculate restricted residual
     a_resCoarse[2h] = I[h->2h] (rhsFine[h] - L[h](phiFine[h])
  */
  virtual void restrictResidual(LevelData<FArrayBox>& a_resCoarse,
                                LevelData<FArrayBox>& a_phiFine,
                                const LevelData<FArrayBox>& a_rhsFine);

  /**
     correct the fine solution based on coarse correction
     a_phiThisLevel += I[2h->h](a_correctCoarse)
  */
  virtual void prolongIncrement(LevelData<FArrayBox>& a_phiThisLevel,
                                const LevelData<FArrayBox>& a_correctCoarse);

  /*@}*/


  /**
     \name AMRLevelOp functions */
  /*@{*/

  /** returns 1 when there are no coarser AMRLevelOp objects */
  virtual int refToCoarser(){ return 2; }

  /** generate approximate solution to L(a_correction) = a_res */
  virtual void AMRRelaxLevel(LevelData<FArrayBox>& a_correction,
                             const LevelData<FArrayBox>& a_res) { ; }

  /** a_residual = a_rhs - L(a_phi, a_phiFine, a_phiCoarse) */
  virtual void AMRResidual(LevelData<FArrayBox>& a_residual,
                           const LevelData<FArrayBox>& a_phiFine,
                           const LevelData<FArrayBox>& a_phi,
                           const LevelData<FArrayBox>& a_phiCoarse,
    const LevelData<FArrayBox>& a_rhs) { ; }

  /** a_residual = a_rhs - L(a_phi, a_phiCoarse)  */
  virtual void AMRResidualNF(LevelData<FArrayBox>& a_residual,
                             const LevelData<FArrayBox>& a_phi,
                             const LevelData<FArrayBox>& a_phiCoarse,
                             const LevelData<FArrayBox>& a_rhs) { ; }

  /** a_resCoarse = I[h-2h]( a_residual - L(a_correction, a_coarseCorrection)) */
  virtual void AMRRestrict(LevelData<FArrayBox>& a_resCoarse,
                           const LevelData<FArrayBox>& a_residual,
                           const LevelData<FArrayBox>& a_correction,
    const LevelData<FArrayBox>& a_coarseCorrection){ ; }

  /** a_correction += I[2h->h](a_coarseCorrection) */
  virtual void AMRProlong(LevelData<FArrayBox>& a_correction,
                          const LevelData<FArrayBox>& a_coarseCorrection){ ; }

  /** a_residual = a_residual - L(a_correction, a_coarseCorrection) */
  virtual void AMRUpdateResidual(LevelData<FArrayBox>& a_residual,
                                 const LevelData<FArrayBox>& a_correction,
                                 const LevelData<FArrayBox>& a_coarseCorrection){;}

  /*@}*/

protected:
  RealVect                m_dx;
  RealVect                m_dxCrse;
  ProblemDomain           m_domain;
  LevelDataOps<FArrayBox> m_levelOps;
  BCFunc                  m_bc;
  LayoutData<NewCFIVS>    m_loCFIVS[SpaceDim];
  LayoutData<NewCFIVS>    m_hiCFIVS[SpaceDim];
  Copier                  m_exchangeCopier;
  QuadCFInterp            m_quadCFI; // left undefined in with regular multigrid define

  void levelGSRB(LevelData<FArrayBox>& a_e,  const LevelData<FArrayBox>& a_residual);
  void homogeneousCFInterp(LevelData<FArrayBox>& a_phif);
  void homogeneousCFInterp(LevelData<FArrayBox>& a_phif,
                           const DataIndex& a_datInd,
                           int a_idir,
                           Side::LoHiSide a_hiorlo);
  void interpOnIVSHomo(LevelData<FArrayBox>& a_phif,
                       const DataIndex& a_datInd,
                       const int a_idir,
                       const Side::LoHiSide a_hiorlo,
                       const IntVectSet& a_interpIVS);

};




class AMRPoissonOpFactory: public AMRLevelOpFactory<LevelData<FArrayBox> >
{
public:
  virtual ~AMRPoissonOpFactory(){;}

  /// full AMR definition function
  void define(const ProblemDomain& a_domain,
              const Vector<DisjointBoxLayout>& a_grids,
              const Vector<int> a_refRatios,
              const RealVect& a_finedx, 
              BCFunc a_bc);


  /// regular multigrid definition function
  void define(const ProblemDomain& a_domain,
              const DisjointBoxLayout& a_grid,
              const RealVect& a_dx, 
              BCFunc a_bc,
              int maxDepth = -1);

  /**
     \name MGLevelOpFactory functions */
  /*@{*/
  virtual AMRPoissonOp*
  MGnewOp(const ProblemDomain& a_FineindexSpace,
                int depth,
                bool homoOnly = true);
 
  /*@}*/

  /**
     \name AMRLevelOpFactory functions */
  /*@{*/
  virtual  AMRPoissonOp* AMRnewOp(const ProblemDomain& a_indexSpace);
 

private:
  Vector<ProblemDomain>     m_domains;
  Vector<DisjointBoxLayout> m_boxes;
  Vector<RealVect>  m_dx;
  BCFunc  m_bc;

};





#endif
