/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// BVS, July 10, 2003

#ifndef _BCFUNC_H_
#define _BCFUNC_H_

#include "FArrayBox.H"
#include "ProblemDomain.H"

typedef void(*BCFunc)(FArrayBox& a_state,
					  const ProblemDomain& a_domain,
					  Real a_dx,
					  bool a_homogeneous);

extern "C" {

// given
//  pos [x,y,z] position on center of cell edge
//  int dir direction, x being 0
//  int side -1 for low, +1 = high,
//  fill in the a_values array

 typedef void(*BCValueFunc)(Real* pos,
                             int* dir,
                             int* side,
                             Real* a_values,
                             int*  a_numValues);


// given
//  pos [x,y,z] position on center of cell edge
//  int dir direction, x being 0
//  int side -1 for low, +1 = high,
//  int homogeneous.  false=0, true=1
//  Real* l1 (first valid point in FArrayBox) and l2,
//      fill in value of 'ghost'

  typedef void(*GenBCFunc)(Real* pos,
                           int* dir,
                           int* side,
                           int* homogeneous,
                           Real* l1,
                           Real* l2,
                           Real* ghost);
}

void GenBC(FArrayBox& a_state,
           const Box& a_valid,
           Real a_dx,
           bool a_homogeneous,
           GenBCFunc a_func,
           int a_dir,
           Side::LoHiSide a_side);

void NeumBC(FArrayBox& a_state,
            const Box& a_valid,
            Real a_dx,
            bool a_homogeneous,
            BCValueFunc a_value,
            int a_dir,
            Side::LoHiSide a_side);

void DiriBC(FArrayBox& a_state,
            const Box& valid,
            Real a_dx,
            bool a_homogeneous,
            BCValueFunc a_value,
            int a_dir,
            Side::LoHiSide a_side);

void DiriBC(FArrayBox& a_state,
            const Box& valid,
            Real a_dx,
            bool a_homogeneous,
            BCValueFunc a_value);


#endif
