/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// BVS, June 18, 2003

// We can assume that template class T has null construction.

#ifndef _LINEARSOLVER_H_
#define _LINEARSOLVER_H_

#include "REAL.H"
#include "Box.H"
#include <cmath>

template <class T>
class LinearOp
{
public:
  virtual ~LinearOp(){;}
  virtual void residual(  T& a_lhs, const T& a_phi, const T& a_rhs, bool a_homogeneous = false) = 0;
  virtual void preCond(   T& a_cor, const T& a_residual)       = 0;
  virtual void applyOp(   T& a_lhs, const T& a_phi, bool a_homogeneous = false) = 0;
  virtual void create(    T& a_lhs, const T& a_rhs) = 0;
  virtual void assign(    T& a_lhs, const T& a_rhs)       = 0;
  virtual Real dotProduct(const T& a_1, const T& a_2)     = 0;
  virtual void incr  (    T& a_lhs, const T& a_x, Real a_scale) = 0;
  virtual void axby(      T& a_lhs, const T& a_x, const T& a_y, Real a_a, Real a_b) = 0;
  virtual void scale(     T& a_lhs, const Real& a_scale)  = 0;
  /// a_ord == 0  max norm, a_ord == 1 sum(abs(a_rhs)), else, L(a_ord) norm.
  virtual Real norm(const T& a_rhs, int a_ord) = 0;
  virtual void setToZero(T& a_lhs) = 0;

};


template <class T>
class LinearSolver
{
public:
  virtual ~LinearSolver(){;}
  virtual void setHomogeneous(bool a_homogeneous) = 0;
  //  You do not own this pointer.
  virtual void define(LinearOp<T>* a_operator, bool a_homogeneous = false) = 0;
  virtual void solve(T& a_phi, const T& a_rhs) = 0;
};



#endif /*_LINEARSOLVER_H_*/
