/*
   __  ______________              __      
  /  |/  / __/ ___/ /  ___  __ _  / /  ___ 
 / /|_/ / _// /__/ _ \/ _ \/  ' \/ _ \/ _ \
/_/  /_/_/  \___/_//_/\___/_/_/_/_.__/\___/
                                           
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _MFCELLFAB_H_
#define _MFCELLFAB_H_

#include "EBCellFAB.H"
#include "BoxLayoutData.H"
#include "EBISLayout.H"
#include "Vector.H"
#include "DisjointBoxLayout.H"
#include "MFIndexSpace.H"

class MFCellFAB
{

public:


  MFCellFAB(){;}

  virtual ~MFCellFAB();

  MFCellFAB(const Vector<EBISBox>& a_phaseGraphs,
            const Box& a_region, const Vector<int>& a_nVar);

  void 
  setVal(Vector<Real> a_value);

  ///
  int nComp(int a_phase) const 
  {
    return (*m_phase[a_phase]).nComp();
  }

  ///
  const Box& box() const {return m_box;}

  EBCellFAB& getPhase(int a_phase)
  {
    return (*m_phase[a_phase]);
  }

  EBCellFAB* getPhasePtr(int a_phase) 
  {
    return m_phase[a_phase];
  }

  ///
  void copy(const Box& RegionFrom, 
            const Interval& destInt, 
            const Box& RegionTo,
            const MFCellFAB& source, 
            const Interval& srcInt);

  static int preAllocatable(){ return 1;}

 ///
  int size(const Box& R, const Interval& comps) const ;
 ///
  void linearOut(void* buf, const Box& R, const Interval& comps) const ;

  ///
  void linearIn(void* buf, const Box& R, const Interval& comps);

  ///invalid but necessary for leveldata to compile
  MFCellFAB(const Box& a_region, int a_nVar)
  {
    MayDay::Error("invalid constructor called for baseebcellfab");
  }

private:

  void operator=(const MFCellFAB& rhs); //unimplemented

  Vector<EBCellFAB*> m_phase;
  Box                m_box;

};


/// Factory class to produce MFCellFABs.
/**
   Factory class to produce MFCellFABs.
   This is needed for LevelData construction.
 */
class MFCellFactory
  : public DataFactory<MFCellFAB>
{
public:

  MFCellFactory(Vector<EBISLayout>& a_ebis, const Vector<int>& a_ncomp);

  MFCellFactory(const MFIndexSpace& a_mf, const DisjointBoxLayout& a_dbl,
                const Box& a_domain, const Vector<int>& a_ncomps,
                int ghost);
                
  void define(Vector<EBISLayout>& a_ebis, const Vector<int>& a_ncomp);

  virtual ~MFCellFactory();
  /// factory function.
  /** 
      Creates a new mfcellfab object 
      and returns a pointer to it.  Responsibility
      for calling operator 'delete' on this pointer is passed to the user. 
  */
  virtual MFCellFAB* create(const Box& a_box, int a_ncompsIgnored,
                            const DataIndex& a_dit) const;

  const EBISLayout& getEBISLayout(int phase) const
  {
    return m_ebis[phase];
  }

private:

  Vector<EBISLayout> m_ebis;
  Vector<int>        m_ncomp;

};

#endif
