/*
   __  ______________              __      
  /  |/  / __/ ___/ /  ___  __ _  / /  ___ 
 / /|_/ / _// /__/ _ \/ _ \/  ' \/ _ \/ _ \
/_/  /_/_/  \___/_//_/\___/_/_/_/_.__/\___/
                                           
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//


#ifndef _MFINDEXSPACE_H_
#define _MFINDEXSPACE_H_

#include "EBIndexSpace.H"

class MFIndexSpace
{
public:

  MFIndexSpace();

  ~MFIndexSpace();

  ///
  /**
     a_geoservers contains a GeometryService per fluid
     phase. 

      If a_ncellMax is set, that is the max width of
      an internal grid.  Otherwise use defaults
      of (16 in 3D, 64 in 2d)
   */
  void
  define(const Box& a_domain,
         const RealVect& a_origin,
         const Real& a_dx,
         const Vector<GeometryService*>& a_geoservers,
         int a_nCellMax = -1,
		 int maxCoarsenings = -1);

  ///
  void fillEBISLayout(Vector<EBISLayout>& a_ebis,
                      const DisjointBoxLayout& a_grids,
                      const Box& a_domain,
                      const int & nghost) const;
  
  void fillEBISLayout(EBISLayout& a_ebis,  int phase,
                      const DisjointBoxLayout& a_grids,
                      const Box& a_domain,
                      const int & nghost) const;

  int numPhases() const {return m_ebis.size();}

  const EBIndexSpace* const EBIS(int a_phase) const {return m_ebis[a_phase];}


  /// 
  /** 
      returns an IntVectSet that is the union of all the irregular 
      cells in the entire MFIndexSpace.  Useful for initializing
      a multifluid calculation.
  */
  IntVectSet interfaceRegion(int a_depth=0) const;

private:

  Vector<EBIndexSpace*> m_ebis;

};

#endif
