

#include "MFIndexSpace.H"


MFIndexSpace::MFIndexSpace() {;}

MFIndexSpace::~MFIndexSpace()
{
  for(int i=0; i<m_ebis.size(); i++){
    delete m_ebis[i];
  }
}


void
MFIndexSpace::define(const Box& a_domain,
                     const RealVect& a_origin,
                     const Real& a_dx,
                     const Vector<GeometryService*>& a_geoservers,
                     int a_nCellMax,
                     int maxCoarsenings)
{
  for(int i=0; i<m_ebis.size(); i++){
    delete m_ebis[i];
  }
  int m_nCellMax = a_nCellMax;
  if(a_nCellMax > 0)
    {
  
    }
  else
    {
      if(SpaceDim == 2)
        {
          m_nCellMax = 64;
        }
      else
        {
          m_nCellMax = 16;
        }
    }
  m_ebis.resize(a_geoservers.size());
  Vector<EBISLevel*> elevels(a_geoservers.size(), NULL);
  int phases = a_geoservers.size();
  for (int i=0; i<phases; i++)
    {
      m_ebis[i] = new EBIndexSpace();
      m_ebis[i]->setCellMax(m_nCellMax);
      assert(a_geoservers[i] != NULL);
      elevels[i] = new EBISLevel(a_domain,a_origin,a_dx,
                                 *(a_geoservers[i]), m_ebis[i]);
      elevels[i]->m_phase = i;
    }
  
  for(int i=0; i<phases; ++i)
    {
      for(int ii=i+1; ii<phases; ++ii)
        {
          elevels[i]->fineLevelStitch(*(elevels[ii]));
        }
      m_ebis[i]->define(elevels[i], m_nCellMax, maxCoarsenings);
      //do NOT delete elevels[i], it has been handed off to EBIndexSpace.
    }
}


void MFIndexSpace::fillEBISLayout(Vector<EBISLayout>& a_ebis,
                                  const DisjointBoxLayout& a_grids,
                                  const Box& a_domain,
                                  const int & nghost) const
{
  assert(a_ebis.size() == m_ebis.size());
  for(int i=0; i<m_ebis.size(); i++){
    m_ebis[i]->fillEBISLayout(a_ebis[i], a_grids,a_domain,nghost);
  }
}
  

void MFIndexSpace::fillEBISLayout(EBISLayout& a_ebis, int phase,
                                  const DisjointBoxLayout& a_grids,
                                  const Box& a_domain,
                                  const int & nghost) const
{

    m_ebis[phase]->fillEBISLayout(a_ebis, a_grids,a_domain,nghost);

}

IntVectSet
MFIndexSpace::interfaceRegion(int depth) const
{
  IntVectSet rtn;
  for(int i=0; i<m_ebis.size(); i++)
    {
      rtn |= m_ebis[i]->irregCells(depth);
    }
  return rtn;
}
