/*   _______              __
    / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL

#ifndef _BINFABFACTORY_H_
#define _BINFABFACTORY_H_

#include "BoxLayoutData.H"
#include "BinFab.H"
#include "DataIndex.H"
#include "ProblemDomain.H"

/// Factory class to produce one or more instances of \p BinFab on the same level.
/**
    The mesh and \p ProblemDomain will be the same for all instances created
    by the factory.  This is needed for LevelData construction.
 */
template <class T>
class BinFabFactory : public DataFactory<BinFab <T> >
{
public:
  /// Factory function.
  /**
      Creates a new \p BinFab object and returns a pointer to it.
      Responsiblitly for calling operator 'delete' on this pointer
      is passed to the user.
      
      Assumes \a a_ncomps is 1 and ignores \a a_dit.
  */
  virtual BinFab<T>* create(const Box& a_box, int a_ncomps,
                            const DataIndex& a_dit) const;

  /// Use these parameters for all manufactured \p BinFab instances.
  BinFabFactory(const RealVect& a_mesh_spacing,
                const RealVect& a_origin,
                const ProblemDomain& a_probdomain);

  /// Destructor
  virtual ~BinFabFactory();

private:
  /// cell size in physical space
  RealVect m_mesh_spacing;

  /// location in physical space of the 0 index
  RealVect m_origin;

  /// contains the \p Box on which this \p BinFab is defined
  ProblemDomain m_probdomain;
};

// Implementation
#include "BinFabFactoryI.H"

#endif
