#ifndef _AndyChannel_H_
#define _AndyChannel_H_

#include "MayDay.H"
#include "LoHiSide.H"
#include "IntVect.H"
#include "RealVect.H"
#include "BaseLevelSet.H"
#include "REAL.H"
#include "BoxIterator.H"

#include <cmath>

using std::pair;

class AndyChannel : public BaseLevelSet
{
public:

  // constructor
  AndyChannel(const Real& a_hW,
	  const Real& a_hw,
	  const Real& a_hd,
	  const Real& a_R,
	  const Real& a_r,
	  const RealVect& a_c);

  // destructor
  virtual ~AndyChannel();

  // return true if all cells are regular (fluid)
  virtual bool isRegular(const Box&      region,
			 const Box&      domain,
			 const RealVect& origin,
			 const Real&     dx) const;

  // return true if all cells are covered (boundary)
  virtual bool isCovered(const Box&      region,
			 const Box&      domain,
			 const RealVect& origin,
			 const Real&     dx) const;

  //
  virtual pair<int, Side::LoHiSide> upDirection(const RealVect& a_midpt,
						const IntVect&  a_iv) const;

  //
  virtual Real localFuncValue(const RealVect& a_independentCoords,
			      const int&      a_updirection,
			      const IntVect&  a_iv,
			      const Box&      a_domain,
			      const RealVect& a_origin,
			      const Real&     a_dx) const;

  // make a new object
  virtual BaseLevelSet* new_baseLevelSet() const;

protected:

private:
  // returns the coordinate of the closest y value for a given (x,y)
  Real yofx(const RealVect& vec) const;
  // returns the coordinate of the closest x value for a given (x,y)
  Real xofy(const RealVect& vec) const;

  Real ABS(const Real& num) const;

  Real m_hW;
  Real m_hw;
  Real m_hd;
  Real m_R;
  Real m_r;
  RealVect m_c;
};

#endif
