#ifndef _Channel_LS_H_
#define _Channel_LS_H_

#include "MayDay.H"
#include "LoHiSide.H"
#include "IntVect.H"
#include "RealVect.H"
#include "BaseLevelSet.H"
#include "REAL.H"
#include "BoxIterator.H"

using std::pair;

class ChannelLS : public BaseLevelSet
{
public:

  // make the object
  ChannelLS(const Real& a_halfbigwidth,
                const Real& a_halfsmallwidth,
                const Real& a_halfdepth,
                const Real& a_bigradius,
                const Real& a_smallradius,
                const Real& a_zradius,
                const RealVect& a_center);

  // destroy the object
  virtual ~ChannelLS();

  // return "true" if all cells in "region" contain fluid
  // and only fluid.  "false" otherwise.
  virtual bool isRegular(const Box&      region,
                         const Box&      domain,
                         const RealVect& origin,
                         const Real&     dx) const;

  // return "true" if no cells in "region" contain any
  // fluid.  "false" otherwise.
  virtual bool isCovered(const Box&      region,
                         const Box&      domain,
                         const RealVect& origin,
                         const Real&     dx) const;

  // at coordinate "a_midpt" associated with cell "a_iv",
  // return:
  //  pair.first = 0,1,2 according to whether x,y,z is cardinal
  //               direction along which distance from "a_midpt"
  //               to the interface is smallest.
  //  pair.second = Side::Lo if the fluid is in the -pair.first direction,
  //                Side::Hi if the fluid is in the +pair.first direction.
  virtual pair<int, Side::LoHiSide> upDirection(
           const RealVect& a_midpt,
           const IntVect&  a_iv) const;

  // given a coordinate "a_independentCoords" in which the
  // "a_updirection" coordinate is considered dependent
  // (the others being independent).  Compute the coordinate
  // of the interface in the dependent direction, fixing
  // the independent coordinates.
  virtual Real localFuncValue(const RealVect& a_independentCoords,
                              const int&      a_updirection,
                              const IntVect&  a_iv,
                              const Box&      a_domain,
                              const RealVect& a_origin,
                              const Real&     a_dx) const;

  // make a new object
  virtual BaseLevelSet* new_baseLevelSet() const;

protected:

private:
  pair<Real,bool> yofxz(const Real& x, const Real& z) const;
  pair<Real,bool> xofyz(const Real& y, const Real& z) const;
  pair<Real,bool> zofxy(const Real& x, const Real& y) const;

  Real m_halfbigwidth;
  Real m_halfsmallwidth;
  Real m_halfdepth;
  Real m_bigradius;
  Real m_smallradius;
  Real m_zradius;
  RealVect m_center;
};

#endif
