/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _COMPLEMENTIF_H_
#define _COMPLEMENTIF_H_

#include "MayDay.H"
#include "RealVect.H"
#include "Vector.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function is the negative of the implicit function with which
    it is constructed.  This results in an implicit function whose interior is
    the complement of the interior of the given implicit function (minus the
    the surface).
 */
class ComplementIF: public BaseImplicitFunction
{
public:
  ///
  /**
      Constructor specifying the implicit function and whether to complement
      (default = true)
   */
  ComplementIF(const BaseImplicitFunction& a_impFunc,
               const bool&                 a_complement = true);

  /// Copy constructor with complement boolean (default = true)
  ComplementIF(const ComplementIF& a_inputIF,
               const bool&         a_complement = true);

  /// Destructor
  virtual ~ComplementIF();

  ///
  /**
      Return the parameter information
   */
  virtual void GetParams(bool& a_complement) const;

  ///
  /**
      Set the parameter information
   */
  virtual void SetParams(const bool& a_complement);

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  bool m_complement;               // complement the function (default = true)

  BaseImplicitFunction* m_impFunc; // implicit function to complement

private:
  ComplementIF()
  {
    MayDay::Error("ComplementIF uses strong construction");
  }

  void operator=(const ComplementIF& a_inputIF)
  {
    MayDay::Error("ComplementIF doesn't allow assignment");
  }
};

#endif
