/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "ComplementIF.H"

ComplementIF::ComplementIF(const BaseImplicitFunction& a_impFunc,
                           const bool&                 a_complement)
{
  m_impFunc = a_impFunc.newImplicitFunction();
  m_complement = a_complement;
}

ComplementIF::ComplementIF(const ComplementIF& a_inputIF,
                           const bool&         a_complement)
{
  m_impFunc = a_inputIF.m_impFunc->newImplicitFunction();
  m_complement = a_complement;
}

ComplementIF::~ComplementIF()
{
  delete m_impFunc;
}

void ComplementIF::GetParams(bool& a_complement) const
{
  // Copy parameter information over
  a_complement = m_complement;
}

void ComplementIF::SetParams(const bool& a_complement)
{
  // Set parameter information
  m_complement = a_complement;
}

Real ComplementIF::value(const RealVect& a_point) const
{
  // Implicit function value
  Real retval;

  // Implicit function value
  retval = m_impFunc->value(a_point);

  // Return the negative if complement is turned on (true)
  if (m_complement)
  {
    retval = -retval;
  }

  return retval;
}

BaseImplicitFunction* ComplementIF::newImplicitFunction() const
{
  ComplementIF* complementPtr = new ComplementIF(*m_impFunc,m_complement);

  return static_cast<BaseImplicitFunction*>(complementPtr);
}
