/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _INTERSECTIONIF_H_
#define _INTERSECTIONIF_H_

#include "MayDay.H"
#include "RealVect.H"
#include "Vector.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function is the maximum of two (of more) implicit
    functions with which it is constructed.  This results in an implicit
    function whose interior is the intersection of the interiors of the given
    implicit functions.
 */
class IntersectionIF: public BaseImplicitFunction
{
public:
  ///
  /**
      Constructor specifying two implicit functions as inputs.
   */
  IntersectionIF(const BaseImplicitFunction& a_impFunc1,
                 const BaseImplicitFunction& a_impFunc2);

  ///
  /**
      Constructor specifying any number of implicit functions as inputs.
   */
  IntersectionIF(const Vector<BaseImplicitFunction*>& a_impFuncs);

  /// Copy constructor
  IntersectionIF(const IntersectionIF& a_inputIF);

  /// Destructor
  virtual ~IntersectionIF();

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  int                           m_numFuncs; // number of implicit functions
  Vector<BaseImplicitFunction*> m_impFuncs; // implicit functions

private:
  IntersectionIF()
  {
    MayDay::Error("IntersectionIF uses strong construction");
  }

  void operator=(const IntersectionIF& a_inputIF)
  {
    MayDay::Error("IntersectionIF doesn't allow assignment");
  }
};

#endif
