/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
//  ANAG, LBNL, DTG

#ifndef _OvoidBL_H_
#define _OvoidBL_H_

#include "MayDay.H"
#include "LoHiSide.H"
#include "IntVect.H"
#include "RealVect.H"
#include "BaseLevelSet.H"
#include "REAL.H"
#include "BoxIterator.H"
using std::pair;

/// 
/*
*/
class OvoidBL: public BaseLevelSet
{
public:

  ///
  OvoidBL(const Real&     a_radius, 
          const RealVect& a_centerLo,
          const Real&     a_axialDist,
          const int&      a_axis,
          const Real&     a_dx);

  ///
  virtual ~OvoidBL();


  ///  
  virtual bool isRegular(const Box&      region,
                         const Box&      domain,
                         const RealVect& origin,
                         const Real&     dx) const;

  /// 
  virtual bool isCovered(const Box&           region,
                         const Box&           domain,
                         const RealVect&      origin,
                         const Real&          dx) const;

  ///
  virtual
  pair<int, Side::LoHiSide>  
  upDirection(const RealVect& a_midpt,const IntVect& a_iv)  const; 

  ///
  virtual Real localFuncValue(const RealVect& a_independentCoords, 
                              const int& a_upDirection,
                              const IntVect& a_iv, 
                              const Box& a_domain, 
                              const RealVect& a_origin, 
                              const Real& a_dx) const;  

 
  
  
  virtual
  BaseLevelSet* new_baseLevelSet() const;   
  // calculates the max coordiante of a RealVect
  virtual Real max(const RealVect& vec)const;
  // calculates the magnitude of a RealVect
  virtual Real magnitude(const RealVect& vec)const;

  void distToSpheres(Real& a_distToLo,
                     Real& a_distToHi,
                     const RealVect& a_vec) const;

  void distToAxis(Real& a_distToAxis,
                  const RealVect& a_vec) const;
protected:
  
  RealVect         m_centerLo;
  RealVect         m_centerHi;
  Real             m_radius;
  Real             m_axialDist;
  int              m_axis;
  Real             m_dx;


private:
  ///
  OvoidBL()
  {
    MayDay::Error("OvoidBL uses strong construction");
  }
  OvoidBL(const OvoidBL& a_radius)
  {
    MayDay::Error("sphereBaselevel disallows copy construction");
  }
  void operator=(const OvoidBL& a_radius)
  {
    MayDay::Error("OvoidBL disallows assignment");
  }

};
#endif
