/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _PLANEIF_H_
#define _PLANEIF_H_

#include "MayDay.H"
#include "RealVect.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function specifies a plane.
 */
class PlaneIF: public BaseImplicitFunction
{
public:
  ///
  /**
      Constructor specifying plane normal (a_normal), a point on the plance
      (a_point), and whether the domain is the same side as the normal
      (a_inside).
   */
  PlaneIF(const RealVect& a_normal,
          const RealVect& a_point,
          const bool&     a_inside);

  /// Copy constructor
  PlaneIF(const PlaneIF& a_inputIF);

  /// Destructor
  virtual ~PlaneIF();

  ///
  /**
      Return the parameter information
   */
  virtual void GetParams(RealVect& a_normal,
                         RealVect& a_point,
                         bool&     a_inside) const;

  ///
  /**
      Set the parameter information
   */
  virtual void SetParams(const RealVect& a_normal,
                         const RealVect& a_point,
                         const bool&     a_inside);

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  RealVect m_normal;    // plane normal
  RealVect m_point;     // point on the plane
  bool     m_inside;    // inside flag

private:
  PlaneIF()
  {
    MayDay::Error("PlaneIF uses strong construction");
  }

  void operator=(const PlaneIF& a_inputIF)
  {
    MayDay::Error("PlaneIF doesn't allow assignment");
  }
};

#endif
