/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "PlaneIF.H"

PlaneIF::PlaneIF(const RealVect& a_normal,
                 const RealVect& a_point,
                 const bool&     a_inside)
{
  // Remember the parameters
  m_normal = a_normal;
  m_point = a_point;
  m_inside = a_inside;
}

PlaneIF::PlaneIF(const PlaneIF& a_inputIF)
{
  // Remember the parameters
  m_normal = a_inputIF.m_normal;
  m_point = a_inputIF.m_point;
  m_inside = a_inputIF.m_inside;
}

PlaneIF::~PlaneIF()
{
}

void PlaneIF::GetParams(RealVect& a_normal,
                        RealVect& a_point,
                        bool&     a_inside) const
{
  // Copy parameter information over
  a_normal = m_normal;
  a_point = m_point;
  a_inside = m_inside;
}

void PlaneIF::SetParams(const RealVect& a_normal,
                        const RealVect& a_point,
                        const bool&     a_inside)
{
  // Set parameter information
  m_normal = a_normal;
  m_point = a_point;
  m_inside = a_inside;
}

Real PlaneIF::value(const RealVect& a_point) const
{
  Real retval;

  // Vector from a_point to a point on the plane
  RealVect direction = m_point;
  direction -= a_point;

  // Compute (and return) the dot product of the vector above and the normal
  retval = 0.0;
  for (int idir = 0; idir < SpaceDim; idir++)
  {

    retval += direction[idir] * m_normal[idir];
  }

  // Change the sign to change inside to outside
  if (!m_inside)
  {
    retval = -retval;
  }

  return retval;
}

BaseImplicitFunction* PlaneIF::newImplicitFunction() const
{
  PlaneIF* planePtr = new PlaneIF(m_normal,
                                  m_point,
                                  m_inside);

  return static_cast<BaseImplicitFunction*>(planePtr);
}
