/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _POLARIF_H_
#define _POLARIF_H_

#include "MayDay.H"
#include "RealVect.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function specifies a cylinder in polar coordinates: 
    r = a_primaryRadius + a_perturbation*cos(a_frequency*theta)
 */
class PolarIF: public BaseImplicitFunction
{
public:
  ///
  PolarIF(const Real& a_primaryRadius,
          const Real& a_perturbation,
          const int & a_frequency,
          const bool& a_inside);

  /// Copy constructor
  PolarIF(const PolarIF& a_inputIF);

  /// Destructor
  virtual ~PolarIF();

  ///
  /**
      Return the parameter information
   */
  virtual void GetParams(Real& a_primaryRadius,
                         Real& a_perturbation,
                         int & a_frequency,
                         bool& a_inside)const;
                        
  ///
  /**
      Set the parameter information
   */
  virtual void SetParams(const Real& a_primaryRadius,
                         const Real& a_perturbation,
                         const int & a_frequency,
                         const bool& a_inside);

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  Real m_primaryRadius;
  Real m_perturbation;
  int  m_frequency;
  bool m_inside;

private:
  PolarIF()
  {
    MayDay::Error("PolarIF uses strong construction");
  }

  void operator=(const PolarIF& a_inputIF)
  {
    MayDay::Error("PolarIF doesn't allow assignment");
  }
};

#endif
