/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "PolarIF.H"

PolarIF::PolarIF(const Real& a_primaryRadius,
                 const Real& a_perturbation,
                 const int & a_frequency,
                 const bool& a_inside)
{
  // Remember the parameters
  m_primaryRadius = a_primaryRadius;
  m_perturbation  = a_perturbation;
  m_frequency     = a_frequency;
  m_inside        = a_inside;
}

PolarIF::PolarIF(const PolarIF& a_inputIF)
{
  // Remember the parameters
  m_primaryRadius = a_inputIF.m_primaryRadius;
  m_perturbation  = a_inputIF.m_perturbation;
  m_frequency     = a_inputIF.m_frequency;
  m_inside        = a_inputIF.m_inside;
}

PolarIF::~PolarIF()
{
}

void PolarIF::GetParams(Real& a_primaryRadius,
               Real& a_perturbation,
               int & a_frequency,
               bool& a_inside)const
{
  // Copy parameter information over
  a_primaryRadius = m_primaryRadius;
  a_perturbation  = m_perturbation;
  a_frequency     = m_frequency;
  a_inside        = m_inside;
 }

void PolarIF::SetParams(const Real& a_primaryRadius,
               const Real& a_perturbation,
               const int & a_frequency,
               const bool& a_inside)
{
  // Set the parameters
  m_primaryRadius = a_primaryRadius;
  m_perturbation  = a_perturbation;
  m_frequency     = a_frequency;
  m_inside        = a_inside;
}

Real PolarIF::value(const RealVect& a_point) const
{
  Real retval;

  Real radius;
  Real theta;
  
  radius = sqrt(a_point[0]*a_point[0] + a_point[1]*a_point[1]);
  theta = atan2(a_point[1],a_point[0]);
  
  retval = radius - (m_primaryRadius + m_perturbation*cos(m_frequency*theta));
  
  if (!m_inside)
    {
      retval = -retval;
    }
  return retval;
}

BaseImplicitFunction* PolarIF::newImplicitFunction() const
{
  PolarIF* polarPtr = new PolarIF( m_primaryRadius,
                                   m_perturbation,
                                   m_frequency,
                                   m_inside);

  return static_cast<BaseImplicitFunction*>(polarPtr);
}
