/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _POLYNOMIALIF_H_
#define _POLYNOMIALIF_H_

#include "MayDay.H"
#include "RealVect.H"
#include "Vector.H"

#include "BaseImplicitFunction.H"

///
/**
    This represents one term in a general polynomial
 */
class PolyTerm
{
public:
  ///
  /** 
      Coefficient of this polynomial term
   */
  Real     coef;

  ///
  /** 
      Powers of this polynomial term
   */
  IntVect powers;
};

///
/**
   This implicit function specifies a general polynomial function.
 */
class PolynomialIF: public BaseImplicitFunction
{
public:
  ///
  /**
      Constructor specifying the term composing the polynomial (a_polynomial)
      and whether the domain is on the inside (a_inside), i.e. where the
      function is negative.
   */
  PolynomialIF(const Vector<PolyTerm>& a_polynomial,
               const bool&             a_inside);

  /// Copy constructor
  PolynomialIF(const PolynomialIF& a_inputIF);

  /// Destructor
  virtual ~PolynomialIF();

  ///
  /**
      Return the parameter information
   */
  virtual void GetParams(Vector<PolyTerm>& a_polynomial,
                         bool&             a_inside) const;

  ///
  /**
      Set the parameter information
   */
  virtual void SetParams(const Vector<PolyTerm>& a_polynomial,
                         const bool&             a_inside);

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  Vector<PolyTerm> m_polynomial;  // polynomial (term by term)
  bool             m_inside;      // inside flag

private:
  PolynomialIF()
  {
    MayDay::Error("PolynomialIF uses strong construction");
  }

  void operator=(const PolynomialIF& a_inputIF)
  {
    MayDay::Error("PolynomialIF doesn't allow assignment");
  }
};

#endif
