/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "PolynomialIF.H"

PolynomialIF::PolynomialIF(const Vector<PolyTerm>& a_polynomial,
                           const bool&             a_inside)
{
  // Copy polynomial
  int size = a_polynomial.size();

  m_polynomial.resize(size);

  for (int iterm = 0; iterm < size; iterm++)
  {
    m_polynomial[iterm] = a_polynomial[iterm];
  }

  // Save inside flag
  m_inside = a_inside;
}

PolynomialIF::PolynomialIF(const PolynomialIF& a_inputIF)
{
  // Copy polynomial
  int size = a_inputIF.m_polynomial.size();

  m_polynomial.resize(size);

  for (int iterm = 0; iterm < size; iterm++)
  {
    m_polynomial[iterm] = a_inputIF.m_polynomial[iterm];
  }

  // Save inside flag
  m_inside = a_inputIF.m_inside;
}

PolynomialIF::~PolynomialIF()
{
}

void PolynomialIF::GetParams(Vector<PolyTerm>& a_polynomial,
                             bool&             a_inside) const
{
  // Copy polynomial
  int size = m_polynomial.size();

  a_polynomial.resize(size);

  for (int iterm = 0; iterm < size; iterm++)
  {
    a_polynomial[iterm] = m_polynomial[iterm];
  }

  // Save inside flag
  a_inside = m_inside;
}

void PolynomialIF::SetParams(const Vector<PolyTerm>& a_polynomial,
                             const bool&             a_inside)
{
  // Copy polynomial
  int size = a_polynomial.size();

  m_polynomial.resize(size);

  for (int iterm = 0; iterm < size; iterm++)
  {
    m_polynomial[iterm] = a_polynomial[iterm];
  }

  // Save inside flag
  m_inside = a_inside;
}

Real PolynomialIF::value(const RealVect& a_point) const
{
  Real retval;

  int size = m_polynomial.size();

  // Evaluate the polynomial
  retval = 0.0;
  for (int iterm = 0; iterm < size; iterm++)
  {
    Real cur;

    cur = m_polynomial[iterm].coef;
    for (int idir = 0; idir < SpaceDim; idir++)
    {
      cur *= pow(a_point[idir],m_polynomial[iterm].powers[idir]);
    }

    retval += cur;
  }

  // Change the sign to change inside to outside
  if (!m_inside)
  {
    retval = -retval;
  }

  return retval;
}

BaseImplicitFunction* PolynomialIF::newImplicitFunction() const
{
  PolynomialIF* polynomialPtr = new PolynomialIF(m_polynomial,
                                                 m_inside);

  return static_cast<BaseImplicitFunction*>(polynomialPtr);
}
