/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>

#include "RealVect.H"

#include "SphereIF.H"

SphereIF::SphereIF(const Real&     a_radius,
                   const RealVect& a_center,
                   const bool&     a_inside)
{
  // Remember the parameters
  m_radius = a_radius;
  m_center = a_center;
  m_inside = a_inside;

  // Precompute the radius squared
  m_radius2 = m_radius * m_radius;
}

SphereIF::SphereIF(const SphereIF& a_inputIF)
{
  // Remember the parameters
  m_radius = a_inputIF.m_radius;
  m_center = a_inputIF.m_center;
  m_inside = a_inputIF.m_inside;

  // Precompute the radius squared
  m_radius2 = m_radius * m_radius;
}

SphereIF::~SphereIF()
{
}

void SphereIF::GetParams(Real&     a_radius,
                         RealVect& a_center,
                         bool&     a_inside) const
{
  // Copy parameter information over
  a_radius = m_radius;
  a_center = m_center;
  a_inside = m_inside;
}

void SphereIF::SetParams(const Real&     a_radius,
                         const RealVect& a_center,
                         const bool&     a_inside)
{
  // Set parameter information
  m_radius = a_radius;
  m_center = a_center;
  m_inside = a_inside;

  // Precompute the radius squared
  m_radius2 = m_radius * m_radius;
}

Real SphereIF::value(const RealVect& a_point) const
{
  Real retval;

  // The distance squared for m_center to a_point
  Real distance2;

  // Compute the distance squared
  distance2 = 0.0;
  for (int idir = 0; idir < SpaceDim; idir++)
  {
    Real cur;
    cur = a_point[idir] - m_center[idir];

    distance2 += cur*cur;
  }

  // Return the difference between the sqaures (zero on the sphere)
  retval = distance2 - m_radius2;

  // Change the sign to change inside to outside
  if (!m_inside)
  {
    retval = -retval;
  }

  return retval;
}

BaseImplicitFunction* SphereIF::newImplicitFunction() const
{
  SphereIF* spherePtr = new SphereIF(m_radius,
                                     m_center,
                                     m_inside);

  return static_cast<BaseImplicitFunction*>(spherePtr);
}
