/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#ifndef _TORUSIF_H_
#define _TORUSIF_H_

#include "MayDay.H"
#include "RealVect.H"

#include "BaseImplicitFunction.H"

///
/**
    This implicit function specifies a torus whose axis is parallel to the
    x-axis
 */
class TorusIF: public BaseImplicitFunction
{
public:
  ///
  /**
      Constructor specifying torus major radius (a_majorRadius), minor radius
      (a_minorRadius), center (a_center), and whether the domain is on the
      inside (a_inside).
   */
  TorusIF(const Real&     a_majorRadius,
          const Real&     a_minorRadius,
          const RealVect& a_center,
          const bool&     a_inside);

  /// Copy constructor
  TorusIF(const TorusIF& a_inputIF);

  /// Destructor
  virtual ~TorusIF();

  ///
  /**
      Return the parameter information
   */
  virtual void GetParams(Real&     a_majorRadius,
                         Real&     a_minorRadius,
                         RealVect& a_center,
                         bool&     a_inside) const;

  ///
  /**
      Set the parameter information
   */
  virtual void SetParams(const Real&     a_majorRadius,
                         const Real&     a_minorRadius,
                         const RealVect& a_center,
                         const bool&     a_inside);

  ///
  /**
      Return the value of the function at a_point.
   */
  virtual Real value(const RealVect& a_point) const;

  virtual BaseImplicitFunction* newImplicitFunction() const;

protected:
  Real     m_majorRadius;   // major radius
  Real     m_minorRadius;   // minor radius

  RealVect m_center;        // center

  bool     m_inside;        // inside flag

  Real     m_minorRadius2;  // precomputed minor radius squared

private:
  TorusIF()
  {
    MayDay::Error("TorusIF uses strong construction");
  }

  void operator=(const TorusIF& a_inputIF)
  {
    MayDay::Error("TorusIF doesn't allow assignment");
  }
};

#endif
