/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "LayoutIterator.H"
#include "HDF5.H"
#include "BRMeshRefine.H"
#include "IntVectSet.H"
#include "LoadBalance.H"

using std::cout;
using std::endl;
using std::sqrt;

/// Global variables;
const char *pgmname = "amrMeshBenchMark" ;
const char *indent = "   ", *indent2 = "      " ; 
bool verbose = true ;
Real innerRadius=0.5;
Real outerRadius=0.75;

RealVect center(D_DECL(0.5, 0.5, 0.5));

int numLevels  = 4;
int nref       = 2;


void
setCircleTags(IntVectSet& ivs, const Box& domain)
{

  Real dx = 1.0d0/domain.size(0);

#if CH_SPACEDIM == 2

  Real bottom = center[1] - outerRadius;
  Real top    = center[1] + outerRadius;
  Real j0 = 0;
  Real j1 = dx;
  for(int j=domain.smallEnd(1); j<domain.bigEnd(1); ++j)
	{
	  if(j0 > top || j1 < bottom){;} // do nothing
	  else
		{
		  

		}
	  j0+=dx;
	  j1+=dx;
	}

		

#elif CH_SPACEDIM == 3
      //sphere
      for(int y=-Y; y<=Y; ++y)
      {
        int Z = (int)sqrt((Real)abs(circleR*circleR - x*x - y*y));
        ivs |= center+IntVect(x,y,Z);
        ivs |= center+IntVect(x,y,-Z);
      }
#else
#error amrMeshBenchmark failed: implemented only for 2D and 3D
#endif
    }
  }
}

void
parseTestOptions( int argc ,char* argv[] ) ;

int
test();


int 
main(int argc, char* argv[])
{
#ifdef MPI
  MPI_Init(&argc, &argv);
#endif
  parseTestOptions( argc ,argv ) ;

  if( verbose )
    cout << indent2 << "Beginning " << pgmname << " ..." << endl ;

  ///
  // Run the tests
  ///
  if(test() != 0)
    {
      cout << indent << pgmname <<" failed"<<endl;
    }
  else
    {
      cout << indent << pgmname <<" passed"<<endl;
    }
#ifdef MPI
  MPI_Finalize();
#endif
}

// returns 0 on all tests passed.

int test()
{
  int error;
  HDF5Handle testFile;

  error = testFile.open("meshs.hdf5", HDF5Handle::CREATE);

  if(error != 0)
    {
      if( verbose )
        cout << indent2 << "File creation failed "<<error<<endl;
      return error;
    }

  assert(testFile.isOpen()); 

  
  ProblemDomain domain(IntVect::TheZeroVector(), 128*IntVect::TheUnitVector());
  
  Vector<LevelData<FArrayBox> > data(numLevels);
  Vector<Vector<Box> >          oldBoxes(numLevels), newBoxes(numLevels);
  Vector<IntVectSet>            tags(numLevels);


  for(int lvl=0; lvl<numLevels; lvl++)
  {	
    IntVectSet& ivs = tags[lvl];
	
    setCircleTags(ivs, domain.domainBox());

	domain.refine(2);
  }


}



