/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// dtgraves mon aug 27, 2001

#include "RampBL.H"
#include "EBIndexSpace.H"
#include "EBISLayout.H"
#include "BoxIterator.H"
#include "ParmParse.H"
#include "BRMeshRefine.H"
#include "LoadBalance.H"
#include "GeometryShop.H"
#include "PolyGeom.H"
#include "LevelData.H"
#include "EBCellFAB.H"
#include "EBCellFactory.H"
#include "VoFIterator.H"
#include "EBArith.H"
#include "AllRegularService.H"
#include "EBPWLFineInterp.H"
#include "DebugDump.H"
#include "SlabService.H"
#include "EBArrayView.H"

/******************/
/******************/
int 
makeLayoutFull(DisjointBoxLayout& dbl1, 
               const Box& domain);
int 
makeLayoutPart(DisjointBoxLayout& dbl1, 
           const Box& domain);
/***************/
/***************/
int makeGeometry(Box& domain, 
                 Real& dx);
/***************/
/***************/
int makeEBISL(EBISLayout& a_ebisl, 
              const DisjointBoxLayout& a_grids, 
              const Box& a_domain, 
              const int& nghost);

/***************/
/***************/
int getError(LevelData<EBCellFAB>& a_errorCoar,
             const EBISLayout& a_ebislFine, 
             const DisjointBoxLayout& a_gridsFine, 
             const Box& a_domainFine,
             const Real& dxFine,
             const EBISLayout& a_ebislCoar,
             const DisjointBoxLayout& a_gridsCoar, 
             const Box& a_domainCoar,
             const Real& dxCoar);
/***************/
/***************/
int compareError(const LevelData<EBCellFAB>& a_fineError,
                 const EBISLayout& a_ebislFine, 
                 const DisjointBoxLayout& a_gridsFine, 
                 const Box& a_domainFine,
                 const LevelData<EBCellFAB>& a_coarError,
                 const EBISLayout& a_ebislCoar, 
                 const DisjointBoxLayout& a_gridsCoar, 
                 const Box& a_domainCoar);

/***************/
/***************/
Real exactFunc(const IntVect& a_iv, 
               const Real& a_dx);
/***************/
/***************/
void dumpmemoryatexit();
/***************/
/***************/
int
main(int argc, char** argv)

{

#ifdef MPI
  MPI_Init(&argc, &argv);
#endif
  int eekflag = 0;
  {//begin forever present scoping trick

    const char* in_file = "pwlinterp.inputs";
    //parse input file
    ParmParse pp(0,NULL,NULL,in_file);
    //define the geometry object.
    //need the new and delete because of
    //strong construction
    //make the gometry.  this makes the first Chombo_EBIS
    Box domainFine, domainMedi, domainCoar;
    Real dxFine, dxMedi, dxCoar;
    //and defines it using a geometryservice
    eekflag =  makeGeometry(domainFine,  dxFine);
    assert(eekflag == 0);

    domainMedi = coarsen(domainFine, 2);
    domainCoar = coarsen(domainMedi, 2);
    dxMedi = 2.0*dxFine;
    dxCoar = 2.0*dxMedi;
    //make grids 
    DisjointBoxLayout gridsFinePart, gridsMediPart;
    DisjointBoxLayout gridsCoarFull, gridsMediFull;

    eekflag = makeLayoutPart(gridsFinePart, domainMedi);
    eekflag = makeLayoutFull(gridsMediFull, domainMedi);
    assert(eekflag == 0);
    coarsen(gridsMediPart, gridsFinePart, 2);
    coarsen(gridsCoarFull, gridsMediFull, 2);

    ///create ebislayout
    int nghost = 0;
    EBISLayout ebislFinePart, ebislMediPart;
    EBISLayout ebislCoarFull, ebislMediFull;

    eekflag = makeEBISL(ebislFinePart, gridsFinePart, domainFine, nghost);
    if(eekflag != 0) return eekflag;
    eekflag = makeEBISL(ebislMediPart, gridsMediPart, domainMedi, nghost);
    if(eekflag != 0) return eekflag;
    eekflag = makeEBISL(ebislMediFull, gridsMediFull, domainMedi, nghost);
    if(eekflag != 0) return eekflag;
    eekflag = makeEBISL(ebislCoarFull, gridsCoarFull, domainCoar, nghost);
    if(eekflag != 0) return eekflag;

    int nvar = 1;
    EBCellFactory ebcellfactFinePart(ebislFinePart);
    EBCellFactory ebcellfactMediPart(ebislMediPart);
    LevelData<EBCellFAB> errorFinePart(gridsFinePart, nvar, 
                                   IntVect::Zero,ebcellfactFinePart);
    LevelData<EBCellFAB> errorMediPart(gridsMediPart, nvar, 
                                   IntVect::Zero,ebcellfactMediPart);
    
    eekflag = getError(errorFinePart, 
                       ebislFinePart, gridsFinePart, domainFine, dxFine,
                       ebislMediFull, gridsMediFull, domainMedi, dxMedi);
    if(eekflag != 0) return eekflag;
    
    eekflag = getError(errorMediPart, 
                       ebislMediPart, gridsMediPart, domainMedi, dxMedi,
                       ebislCoarFull, gridsCoarFull, domainCoar, dxCoar);
    if(eekflag != 0) return eekflag;
    
    eekflag =  compareError(errorFinePart, ebislFinePart, gridsFinePart, domainFine,
                            errorMediPart, ebislMediPart, gridsMediPart, domainMedi);
    if(eekflag != 0) return eekflag;
    
  }//end omnipresent scoping trick
  EBIndexSpace* ebisPtr = Chombo_EBIS::instance();
  ebisPtr->clear();
#ifdef MPI
  MPI_Finalize();
#endif
  return eekflag;
}
/***************/
/***************/
int getError(LevelData<EBCellFAB>& a_errorFine,
             const EBISLayout& a_ebislFine, 
             const DisjointBoxLayout& a_gridsFine, 
             const Box& a_domainFine,
             const Real& a_dxFine,
             const EBISLayout& a_ebislCoar, 
             const DisjointBoxLayout& a_gridsCoar, 
             const Box& a_domainCoar,
             const Real& a_dxCoar)
{
  int eekflag = 0;
  int nvar = 1;
  int nref = 2;

  EBCellFactory ebcellfactFine(a_ebislFine);
  EBCellFactory ebcellfactCoar(a_ebislCoar);
  LevelData<EBCellFAB> phiFine(a_gridsFine, nvar, 
                               IntVect::Zero,ebcellfactFine);
  LevelData<EBCellFAB> phiCoar(a_gridsCoar, nvar, 
                               IntVect::Zero,ebcellfactCoar);

  //fill phi fine and phiCExact
  //put phiFexact into a_errorFine
  for(DataIterator dit = a_gridsFine.dataIterator();
      dit.ok(); ++dit)
    {
      IntVectSet ivsBox(a_gridsFine.get(dit()));
      phiFine[dit()].setVal(0.0);
      EBCellFAB& phiFineFAB = a_errorFine[dit()];
      phiFineFAB.setCoveredCellVal(0.0,0);
      for(VoFIterator vofit(ivsBox, a_ebislFine[dit()].getEBGraph());
          vofit.ok(); ++vofit)
        {
          const VolIndex& vof = vofit();
          Real rightAns = exactFunc(vof.gridIndex(), a_dxFine);
          phiFineFAB(vof, 0) = rightAns;
        }
    }

  for(DataIterator dit = a_gridsCoar.dataIterator();
      dit.ok(); ++dit)
    {
      IntVectSet ivsBox(a_gridsCoar.get(dit()));
      EBCellFAB& phiCoarFAB = phiCoar[dit()];
      phiCoarFAB.setCoveredCellVal(0.0,0);
      for(VoFIterator vofit(ivsBox, a_ebislCoar[dit()].getEBGraph());
          vofit.ok(); ++vofit)
        {
          const VolIndex& vof = vofit();
          Real rightAns = exactFunc(vof.gridIndex(), a_dxCoar);
          phiCoarFAB(vof, 0) = rightAns;
        }
    }

  EBPWLFineInterp interpOp(a_gridsFine, a_gridsCoar,
                           a_ebislFine, a_ebislCoar,
                           a_domainCoar, nref, nvar);

  Interval zeroiv(0,0);
  interpOp.interpolate(phiFine, phiCoar, zeroiv);
  //error = phiC - phiCExact
  for(DataIterator dit = a_gridsFine.dataIterator();
      dit.ok(); ++dit)
    {
      EBCellFAB& errorFAB = a_errorFine[dit()];
      EBCellFAB& phiFineFAB = phiFine[dit()];
      
      errorFAB -= phiFineFAB;
    }

  return eekflag;
}
/***************/
/***************/
int compareError(const LevelData<EBCellFAB>& a_errorFine,
                 const EBISLayout& a_ebislFine, 
                 const DisjointBoxLayout& a_gridsFine, 
                 const Box& a_domainFine,
                 const LevelData<EBCellFAB>& a_errorCoar,
                 const EBISLayout& a_ebislCoar, 
                 const DisjointBoxLayout& a_gridsCoar, 
                 const Box& a_domainCoar)
{
  EBCellFactory factFine(a_ebislFine);
  EBCellFactory factCoar(a_ebislCoar);
  int eekflag = 0;
  for(int itype = 0; itype < 3; itype++)
    {
      EBNormType::NormMode normtype;
      if(itype == 0)
        {
          normtype = EBNormType::OverBoth;
          pout() << endl << " Using all uncovered cells." << endl  << endl;
        }
      else if(itype == 1)
        {
          normtype = EBNormType::OverOnlyRegular;
          pout() << endl << " Using only regular cells." << endl << endl;
        }
      else
        {
          normtype = EBNormType::OverOnlyIrregular;
          pout() << endl << " Using only irregular cells." << endl << endl;
        }
      int comp = 0;
      for(int inorm = 0; inorm < 3; inorm++)
        {

          if(inorm == 0)
            {
              pout() << endl << " Using max norm." << endl;
            }
          else
            {
              pout() << endl << " Using L-" << inorm << "norm." << endl;
            }
          Real coarnorm = EBArith::norm(a_errorCoar, 
                                        a_gridsCoar, a_ebislCoar, 
                                        comp, inorm, normtype);
          Real finenorm = EBArith::norm(a_errorFine, 
                                        a_gridsFine, a_ebislFine, 
                                        comp, inorm, normtype);
          pout() << "Coarse Error Norm = " << coarnorm << endl;
          pout() << "Fine   Error Norm = " << finenorm << endl;
          if(Abs(finenorm) > 1.0e-8)
            {
              Real order = log(coarnorm/finenorm)/log(2.0);
              pout() << "Order of scheme = " << order << endl;
            }
        }
    }

  return eekflag;
}

/***************/
/***************/
Real exactFunc(const IntVect& a_iv, const Real& a_dx)
{
  Real retval;
  Real probHi;
  ParmParse pp;
  pp.get("prob_hi",probHi);
  RealVect xloc;
  for(int idir = 0; idir < SpaceDim; idir++)
    {
      xloc[idir] = a_dx*(Real(a_iv[idir]) + 0.5);
    }
  Real x = xloc[0]/probHi;
  Real y = xloc[1]/probHi;
  
  retval = 1.0 + x*x + y*y + x*x*x + y*y*y;
  //  retval = 2.*xloc[0];
  //  retval  = x;
  //  retval = xloc[0]*xloc[0];
  return retval;
}
/***************/
/***************/
int makeEBISL(EBISLayout& a_ebisl, 
              const DisjointBoxLayout& a_grids, 
              const Box& a_domain, 
              const int& a_nghost)
{
  const EBIndexSpace* const ebisPtr = Chombo_EBIS::instance();
  assert(ebisPtr->isDefined());
  ebisPtr->fillEBISLayout(a_ebisl, a_grids, a_domain, a_nghost);
  return 0;
}
/***************/
/***************/
int 
makeTags(IntVectSet& a_tags, 
         const Box& a_domainCoar)
{
#if(CH_SPACEDIM ==2)
  int nc = a_domainCoar.size(0);
  if(nc < 16)
    {
      pout() << "coarsest level must be at least 16x16" << endl;
      return -22;
    }
  int nmi = nc/2;//16
  int nqu = nc/4;//8
  int ntf = (nc*3)/4;  //24
  int nte = (nc*3)/8; //12
  int nfe = (nc*5)/8; //20
  Box boxf2(IntVect(nmi,nte), IntVect(ntf-1,nfe-1));
  Box boxf4(IntVect(nfe,nqu), IntVect(nc -1,nte-1));
#else
  int nc = a_domainCoar.size(0);
  int nmi = nc/2;//16
  int nqu = nc/4;//8
  int ntf = (nc*3)/4;  //24
  Box boxf2(IntVect(0, nqu,nqu), IntVect(nmi-1,ntf-1,ntf-1));
  Box boxf4(IntVect(0, 0, 0), IntVect(nqu-1,nqu-1,nqu-1));
#endif
  a_tags |= boxf2;
  a_tags |= boxf4;
  return 0;
}
int 
makeLayoutPart(DisjointBoxLayout& a_dbl, 
               const Box& a_domainCoarsest)
{
  Vector<int> vecRefRat(2, 2);
  ParmParse pp;
  int eekflag= 0;
  int blockFactor, bufferSize, maxSize;
  Real fillRat;
  pp.get("block_factor", blockFactor);
  pp.get("buffer_size", bufferSize);
  pp.get("maxboxsize", maxSize);
  pp.get("fill_ratio", fillRat);

  BRMeshRefine mesher(a_domainCoarsest, vecRefRat,
                      fillRat, blockFactor, bufferSize, maxSize);

  int topLevel  =  0;
  int baseLevel =  0;
  int nlevels =  2;
  //tags at base level
  IntVectSet tags;
  eekflag = makeTags(tags, a_domainCoarsest);
  if(eekflag < 0) return eekflag;
  
  Vector<Vector<Box> > oldMeshes(nlevels);
  oldMeshes[0] = Vector<Box>(1, a_domainCoarsest);
  Box finerDomain = a_domainCoarsest;
  for(int ilev = 1; ilev < nlevels; ilev++)
    {
      finerDomain.refine(vecRefRat[ilev]);
      oldMeshes[ilev] = Vector<Box>(1, finerDomain);
    }
  Vector<Vector<Box> > newMeshes;
  mesher.regrid(newMeshes, tags, baseLevel, topLevel, oldMeshes);

  Vector<int> procAssign;
  eekflag = LoadBalance(procAssign, newMeshes[1]);
  if(eekflag != 0) return eekflag;
  a_dbl.define(newMeshes[1], procAssign);
  int iverbose;
  pp.get("verbose", iverbose);
  if(iverbose == 1)
    {
      pout() << "the grids coarser domain= " << a_domainCoarsest 
             << " = " << a_dbl << endl;
    }
      
  return eekflag;
}
/***************/
/***************/
int 
makeLayoutFull(DisjointBoxLayout& a_dbl, 
               const Box& a_domain)
{
  ParmParse pp;
  int eekflag= 0;
  int ipieces;
  ipieces = Max(ipieces, 1);
  int maxsize;
  pp.get("maxboxsize",maxsize);
  Vector<Box> vbox(1, a_domain);
  domainSplit(a_domain, vbox,  maxsize);
  if(eekflag != 0)
    {
      pout() << "problem in domainsplit" << endl;
      return eekflag;
    }
  Vector<int>  procAssign;
  eekflag = LoadBalance(procAssign,vbox);
  if(eekflag != 0)
    {
      pout() << "problem in loadbalance" << endl;
      return eekflag;
    }
  a_dbl.define(vbox, procAssign);
  return eekflag;
}
/**********/
int makeGeometry(Box& a_domain, 
                 Real& a_dx)
{
  int eekflag =  0;
  //parse input file
  ParmParse pp;
  RealVect origin = RealVect::Zero;
#if(CH_SPACEDIM==2)
  Vector<int> n_cell(SpaceDim, 64);
#else
  Vector<int> n_cell(SpaceDim, 16);
#endif

  IntVect lo = IntVect::TheZeroVector();
  IntVect hi;
  for(int ivec = 0; ivec < SpaceDim; ivec++)
    {
      if(n_cell[ivec] <= 0)
        {
          pout() << " bogus number of cells input = " << n_cell[ivec];
          return(-1);
        }
      hi[ivec] = n_cell[ivec] - 1;
    }

  a_domain.setSmall(lo);
  a_domain.setBig(hi);

  Vector<Real> prob_lo(SpaceDim, 1.0);
  Real prob_hi;
  pp.getarr("prob_lo",prob_lo,0,SpaceDim);
  pp.get("prob_hi",prob_hi);
  a_dx = (prob_hi-prob_lo[0])/n_cell[0];
  for(int idir = 0; idir < SpaceDim; idir++)
    {
      origin[idir] = prob_lo[idir];
    }
  int whichgeom;
  pp.get("which_geom",whichgeom);
  if(whichgeom == 0)
    {
      //allregular
      pout() << "all regular geometry" << endl;
      AllRegularService regserv;
      EBIndexSpace* ebisPtr = Chombo_EBIS::instance();
      ebisPtr->define(a_domain, origin, a_dx, regserv);
    }
  else if(whichgeom == 1)
    {
      pout() << "ramp geometry" << endl;
      int upDir;
      int indepVar;
      Real startPt;
      Real slope;
      pp.get("up_dir",upDir);
      pp.get("indep_var",indepVar);
      pp.get("start_pt", startPt);
      pp.get("ramp_slope", slope);

      RampBL ramp(upDir, indepVar, startPt, slope);
      GeometryShop workshop(ramp);
      //this generates the new EBIS
      EBIndexSpace* ebisPtr = Chombo_EBIS::instance();
      ebisPtr->define(a_domain, origin, a_dx, workshop);
    }
  else if(whichgeom == 2)
    {
      pout() << "slab geometry" << endl;
      vector<int> slab_lo(SpaceDim);
      pp.getarr("slab_lo",slab_lo,0,SpaceDim);
      vector<int> slab_hi(SpaceDim);
      pp.getarr("slab_hi",slab_hi,0,SpaceDim);
      IntVect lo, hi;
      for(int idir = 0; idir < SpaceDim; idir++)
        {
          lo[idir] = slab_lo[idir];
          hi[idir] = slab_hi[idir];
        }
      Box coveredBox(lo,hi);
      SlabService slab(coveredBox);
      //this generates the new EBIS
      EBIndexSpace* ebisPtr = Chombo_EBIS::instance();
      RealVect origin = RealVect::Zero;
      ebisPtr->define(a_domain, origin, a_dx, slab);
    }
  else
    {
      //bogus which_geom
      pout() << " bogus which_geom input = " << whichgeom;
      eekflag = 33;
    }

  return eekflag;
}
/***************/
/***************/
