/* Lots of headers.  Lots and lots and lots of headers. */


/* _______              __
   / ___/ /  ___  __ _  / /  ___
   / /__/ _ \/ _ \/  ' \/ _ \/ _ \
   \___/_//_/\___/_/_/_/_.__/\___/ 
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents ofthe University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include <cmath>
#include <cstdlib>
#include <cstring>
#include <cassert>
#include "AllRegularService.H"
#include "Box.H"
#include "BoxIterator.H"
#include "CONSTANTS.H"
#include "DataIterator.H"
#include "DebugDump.H"
#include "EBArrayView.H"
#include "EBCellFAB.H"
#include "EBFaceFAB.H"
#include "EBCellFactory.H"
#include "EBIndexSpace.H"
#include "FArrayBox.H"
#include "LayoutIterator.H"
#include "LevelData.H"
#include "Misc.H"
#include "REAL.H"
#include "SPMD.H"
#include "SlabService.H"
#include "UGIO.H"
#include "Vector.H"

#include "RampBL.H"
#include "GeometryShop.H"


void do_geo( Box &domain, Real &dx );

void fillit( LevelData<EBCellFAB>& tofill );

int
main(int argc, char** argv)
{

#ifdef MPI
  MPI_Init(&argc, &argv);
#endif
  //begin forever present scoping trick
  {
    // Set up some geometry.
    Box domain;
    Real dx;

    int GhostThickness = 2;
  
    do_geo( domain, dx );
  
    // Make a layout for the space.
    DisjointBoxLayout dbl;
    {
      BoxLayout bl;
      Box sbox = domain;
      Box otherbox = sbox.chop(0,5);
      cout << "Box 1 " << sbox << endl;
      cout << "Box 2 " << otherbox << endl;
      bl.addBox(sbox, 0);
      bl.addBox(otherbox, 0);
    
      bl.close();
      dbl.define(bl);
      dbl.close();
    }

    // Fill in the layout with the geometrical info.
    EBISLayout mylayout;
    EBIndexSpace *ebisPtr = Chombo_EBIS::instance();
    ebisPtr->fillEBISLayout( mylayout, dbl, domain, 1 );
  
    // Define the leveldata for my one and only level.
    EBCellFactory ebfact(mylayout);
    LevelData<EBCellFAB> Udata;
    Udata.define(dbl, 4, 
                 GhostThickness*IntVect::TheUnitVector(), ebfact );

    DataIterator dit = dbl.dataIterator();
    for( dit.begin(); dit.ok(); ++dit )
      {
        EBCellFAB& curcf = Udata[dit()];

        BaseFab<Real>& foo= curcf.getSingleValuedFAB();

        cout << foo.box() << endl;      
      }
     
      
  }//end scoping trick
#ifdef MPI
  MPI_Finalize();
#endif
  return 0;
}




void do_geo( Box &domain, Real &dx )
{
  int N = 10;
  Real xdom = 1.0;

  dx = xdom/N;
  const IntVect hi = (N-1)*IntVect::Unit;
  domain=  Box(IntVect::Zero, hi);

  int updir=1;
  int indv=0;
  Real start = 0.5;
  Real slope = tan( M_PI/6.0 );
  RampBL myramp( updir, indv, start, slope );
  GeometryShop mygeom( myramp );

  RealVect origin = RealVect::Zero;
  
  EBIndexSpace *ebisPtr = Chombo_EBIS::instance();

  ebisPtr->define(domain, origin, dx, mygeom);

  return;
}
