/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/
*/
//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
//
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//

#include "AMRLevel.H"
#include "AMRLevelIdealMHD.H"
#include "AMRLevelIdealMHDFactory.H"

AMRLevelIdealMHDFactory::AMRLevelIdealMHDFactory()
{
  setDefaultValues();
}

// Virtual constructor
AMRLevel* AMRLevelIdealMHDFactory::new_amrlevel() const
{
  cout << "AMRLEVELIDEALFACTORY will return new_amrlevel" << endl;
  // Make sure everything is defined
  assert(isDefined());

  // Create a new AMRLevelIdealMHD
  AMRLevelIdealMHD* amrGodPtr = new AMRLevelIdealMHD();

  // Set up new object
  amrGodPtr->CFL(m_cfl);
  amrGodPtr->domainLength(m_domainLength);
  amrGodPtr->refinementThreshold(m_refineThresh);
  amrGodPtr->tagBufferSize(m_tagBufferSize);
  amrGodPtr->initialDtMultiplier(m_initialDtMultiplier);
  amrGodPtr->patchGodunov(m_patchGodunov);
  amrGodPtr->poissonOp(m_poissonOp);
  amrGodPtr->verbosity(m_verbosity);

  // Return it
  return (static_cast <AMRLevel*> (amrGodPtr));
}

AMRLevelIdealMHDFactory::~AMRLevelIdealMHDFactory()
{
}

// CFL number
void AMRLevelIdealMHDFactory::CFL(Real a_cfl)
{
  m_cfl = a_cfl;
  m_cflSet = true;
}

void AMRLevelIdealMHDFactory::verbosity(const int& a_verbosity)
{
  m_verbosity = a_verbosity;
}


// Physical dimension of the longest side of the domain
void AMRLevelIdealMHDFactory::domainLength(Real a_domainLength)
{
  m_domainLength = a_domainLength;
  m_domainLengthSet = true;
}

// Refinement threshold
void AMRLevelIdealMHDFactory::refinementThreshold(Real a_refineThresh)
{
  m_refineThresh = a_refineThresh;
  m_refineThreshSet = true;
}

// Tag buffer size
void AMRLevelIdealMHDFactory::tagBufferSize(int a_tagBufferSize)
{
  m_tagBufferSize = a_tagBufferSize;
  m_tagBufferSizeSet = true;
}

// Initial dt multiplier
void AMRLevelIdealMHDFactory::initialDtMultiplier(Real a_initialDtMultiplier)
{
  m_initialDtMultiplier = a_initialDtMultiplier;
  m_initialDtMultiplierSet = true;
}

// PatchGodunov object (used as a factory)
void AMRLevelIdealMHDFactory::patchGodunov(RefCountedPtr<PatchGodunov> a_patchGodunov)
{
   cout << "In AMRLevelIdealMHDFactory patchGodunov factory ptr addr:" << a_patchGodunov << endl;
  m_patchGodunov = a_patchGodunov;
  m_patchGodunovSet = true;
}

// PoissonOp object (used as a factory)
void AMRLevelIdealMHDFactory::poissonOp(RefCountedPtr<PoissonOp> a_poissonOp)
{
   cout << "In AMRLevelIdealMHDFactory poissonOp factory ptr addr:" << a_poissonOp << endl;
  m_poissonOp = a_poissonOp;
  m_poissonOpSet = true;
}

// Check that everything is defined
bool AMRLevelIdealMHDFactory::isDefined() const
{
  return (m_cflSet &&
          m_domainLengthSet &&
          m_refineThreshSet &&
          m_tagBufferSizeSet &&
          m_initialDtMultiplierSet &&
          m_poissonOp &&
          m_patchGodunovSet);
}

// Some default values
void AMRLevelIdealMHDFactory::setDefaultValues()
{
  CFL(0.3);
  domainLength(2.0);
  refinementThreshold(0.2);
  tagBufferSize(2);
  initialDtMultiplier(0.1);
  m_patchGodunovSet = false;
  m_poissonOpSet = false;
}
